// Class for creating an object that contains general scene and accident
// information

// 	Name: AccidentInfo (class)
//	Author: Arjan Egges
//	Version: 1.0

package  se.lth.cs.carsimgui;


class AccidentInfo {

   private static final int MAX_DYN_OBJ = 20;
   private static final int MAX_STAT_OBJ = 20;
   private static final int MAX_COLL_OBJ = 10;
   
   // Array of dynamic accident objects
   private DynamicAccidentObject[] dynObjArray;
   private int dynAmount;
   
   // Array of static accident objects
   private AccidentObject[] objArray;
   private int stAmount;
   
   // Array of collision objects
   private CollisionInfo[] collArray;
   private int collAmount;
   
   // Array of dynamic info objects
   private DynamicInfo[] dynInfoArray;
   private int dynInfoAmount;
   
   // Constructor
   public AccidentInfo() {
      // Initialize the dynamic object array
      dynAmount = 0;
      dynObjArray = new DynamicAccidentObject[MAX_DYN_OBJ];
      
      // Initialize the static object array
      stAmount = 0;
      objArray = new AccidentObject[MAX_STAT_OBJ];      
      
      // Initialize the dynamic info array
      dynInfoAmount = 0;
      dynInfoArray = new DynamicInfo[MAX_DYN_OBJ];
      
      // Initialize the collision info array
      collAmount = 0;
      collArray = new CollisionInfo[MAX_COLL_OBJ];
      
   }
   
   // Adds a dynamic object to the accident configuration
   public void addDynamicObject(DynamicAccidentObject object) {
      if (dynAmount < MAX_DYN_OBJ) {
         // Add the object to the dynamic object array
         dynObjArray[dynAmount] = object;
         dynAmount++;
      } else
         Messenger.message("Too many dynamic objects. Unable to add object.");
   }
   
   // Returns the dynamic object at index 'index'
   public DynamicAccidentObject getDynamicObject(int index) {
      if (index >= 0 && index < dynAmount)
         return dynObjArray[index];
      else 
         return new SimpleCar();
   }
   
   // Adds a static object to the accident configuration
   public void addStaticObject(AccidentObject object) {
      if (stAmount < MAX_STAT_OBJ) {
         // Add the object to the static object array
         objArray[stAmount] = object;
         stAmount++;
      } else
         Messenger.message("Too many static objects. Unable to add object.");
   }
   
   // Returns the static object at index 'index'
   public AccidentObject getStaticObject(int index) {
      if (index >= 0 && index < stAmount)
         return objArray[index];
      else 
         return new Roadsign();
   }
   
   // Adds a dynamic info object to the accident configuration
   public void addDynamicInfo(DynamicInfo object) {
      if (dynInfoAmount < MAX_DYN_OBJ) {
         // Add the object to the dynamic object array
         //dynInfoArray[dynAmount] = object; the old version from Arjan
         dynInfoArray[dynInfoAmount] = object;
         dynInfoAmount++;
      } else
         Messenger.message("Unable to add DynamicInfo object.");
   }
   
   // Returns the dynamic info object at index 'index'
   public DynamicInfo getDynamicInfo(int index) {
      if (index >= 0 && index < dynAmount)
         return dynInfoArray[index];
      else
         return new DynamicInfo();
   }
   
   // Adds a collision object to the accident configuration
   public void addCollisionInfo(CollisionInfo object) {
      if (collAmount < MAX_COLL_OBJ) {
         // Add the object to the static object array
         collArray[collAmount] = object;
         collAmount++;
      } else
         Messenger.message("Too many collision objects. Unable to add object.");
   }
   
   // Returns the collision object at index 'index'
   public CollisionInfo getCollisionInfo(int index) {
      if (index >= 0 && index < collAmount)
         return collArray[index];
      else 
         return new CollisionInfo();
   }

   // Returns the amount of static objects
   public int getStaticAmount() {
      return stAmount;
   }
   
   // Returns the amount of dynamic objects
   public int getDynamicAmount() {
      return dynAmount;
   }
   
   // Returns the amount of dynamic info objects
   public int getDynInfoAmount() {
      return dynInfoAmount;
   }
   
   // Returns the amount of collision objects
   public int getCollisionInfoAmount() {
      return collAmount;
   }





   // for debugging
   public void printAccidentInfo() {
   	int i;
   	System.out.println("dynAmount: "+dynAmount);
   	for (i=0;i<dynAmount;i++)
   	{
   		System.out.println("("+i+")");
   		System.out.println("id     : "+dynObjArray[i].getId());
   		System.out.println("length : "+dynObjArray[i].getLength());
   		System.out.println("width  : "+dynObjArray[i].getWidth());
   	}
   	System.out.println("-----");
   	System.out.println("stAmount: "+stAmount);
   	for (i=0;i<stAmount;i++)
   	{
   		System.out.println("("+i+")");
   		System.out.println("id     : "+objArray[i].getId());
   		System.out.println("x      : "+objArray[i].getXPos());
   		System.out.println("y      : "+objArray[i].getYPos());
   		System.out.println("radius : "+objArray[i].getCollisionRadius());
   	}
   	System.out.println("-----");
   	System.out.println("dynInfoAmount: "+dynInfoAmount);
   	for (i=0;i<dynInfoAmount;i++)
   	{
   		System.out.println("("+i+")");
	   	System.out.println("startsignIndex : "+dynInfoArray[i].startsignIndex);
	   	System.out.println("endsignIndex   : "+dynInfoArray[i].endsignIndex);
	   	System.out.println("startsignIndex : "+dynInfoArray[i].startsignIndex);
	   	System.out.println("initDirection  : "+dynInfoArray[i].initDirection);
	   	System.out.println("startPoint     : "+dynInfoArray[i].startPoint);
	   	System.out.println("endPoint       : "+dynInfoArray[i].endPoint);
	   	System.out.println("eventAmount    : "+dynInfoArray[i].ceb.getEventAmount());
		for (int j=0; j < dynInfoArray[i].ceb.getEventAmount(); j++)
		{
			System.out.println("event["+j+"]: "+dynInfoArray[i].ceb.getEvent(j).kind);
		}
		System.out.println("DF=0, TL=4, TR=5, STOP=7, CL_L=8, CL_R=9, LR=10, LR_L=11, LR_R=12, OT=13, OT_L=14, OT_R=15");
	}
   	System.out.println("-----");
   	System.out.println("collAmount : "+collAmount);
   	for (i=0;i<collAmount;i++)
   	{
   		System.out.println("("+i+")");
   		System.out.println("actorIndex     : "+collArray[i].actorIndex);
   		System.out.println("victimIndex    : "+collArray[i].victimIndex);
   		System.out.println("victimKind     : "+collArray[i].victimKind);
   		System.out.println("STATIC=0, DYNAMIC=1");
   		System.out.println("actorPart      : "+collArray[i].actorPart);
   		System.out.println("victimPart     : "+collArray[i].victimPart);
   		System.out.println("FRONT=0, REAR=1, LEFTSIDE=2, RIGHTSIDE=3, UNKNOWN_PART=4");
   		System.out.println("actorOverturn  : "+collArray[i].actorOverturn);
   		System.out.println("victimOverturn : "+collArray[i].victimOverturn);
		System.out.println("OT=13, OT_L=14, OT_R=15, NOT=16");
   	}
   	System.out.println("-----");
   }
}