// This is a generic class for creating the accident environment.

// 	Name: AccidentUniverse (class)
//	Author: Arjan Egges
//	Version 1.0

package  se.lth.cs.carsimgui;


import com.sun.j3d.utils.behaviors.mouse.*;
import com.sun.j3d.utils.behaviors.keyboard.*;
import java.awt.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;


class AccidentUniverse {

   // Public universe properties
   public boolean backgroundTexture;
   public boolean floorTexture;
   public boolean createGeo;
   
   // private members
   private BranchGroup objRoot;
   private TransformGroup objTransDown;
   private TransformGroup objScale;
   
   private SimpleUniverse u;
   private Canvas3D c;
   
   // private dynamic variables
   // This object keeps track of the dynami and static objects added to the universe
   private AccidentInfo ainfo;
   
   // Constructor
   public AccidentUniverse() {
      GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();
      c = new Canvas3D(config);  
      c.setSize(600,600);
      NewTextureLoader.setImageObserver(c);
      u = new SimpleUniverse(c);

      // This will move the ViewPlatform back a bit so the
      // objects in the scene can be viewed.
      u.getViewingPlatform().setNominalViewingTransform();
      
      // Create the root of the branch graph
      objRoot = new BranchGroup();
      
      // Create a Transformgroup to translate all the objects down for better viewing.
      Transform3D translate = new Transform3D();
      translate.set(new Vector3f(0.0f, -1.0f, -6.0f));
      objTransDown = new TransformGroup(translate);
      objRoot.addChild(objTransDown);
      
      // Create a Transformgroup to scale all objects so they
      // appear in the scene.
      Transform3D t3d = new Transform3D();
      t3d.setScale(0.3);
      objScale = new TransformGroup(t3d);
      objTransDown.addChild(objScale);    
      
      // Set default values for the textures.
      backgroundTexture = true;
      floorTexture = true;
      createGeo = true;
      
      ainfo = new AccidentInfo();
   }

   // This will create the basic geometry for the virtual environment.
   public void createGeometry() {  
      // Set the properties
      createGeo = Messenger.properties.createEnvironment;
      backgroundTexture = Messenger.properties.textureOn;
      floorTexture = Messenger.properties.textureOn;
      
      TransformGroup viewTrans =
            u.getViewingPlatform().getViewPlatformTransform();        

      // Create the transform group node and initialize it to the identity
      TransformGroup objTrans = new TransformGroup();
      objScale.addChild(objTrans);
      
      // Initialize the background
      BoundingSphere bounds = new BoundingSphere(new Point3d(0.0,0.0,0.0), 400.0);
      if (createGeo) {  
         Background bg = new Background();
         bg.setApplicationBounds(bounds);
         BranchGroup backGeoBranch = new BranchGroup();
         Sphere sphereObj = new Sphere(1.0f, Sphere.GENERATE_NORMALS |
   		          Sphere.GENERATE_NORMALS_INWARD |
			  Sphere.GENERATE_TEXTURE_COORDS, 45);
         Appearance backgroundApp = sphereObj.getAppearance();
         backGeoBranch.addChild(sphereObj);
         bg.setGeometry(backGeoBranch);
         objTrans.addChild(bg);
        
         // Load the texture onto the background sphere
         if (backgroundTexture) {
            NewTextureLoader tex = new NewTextureLoader("images/bg.jpg", "RGB");
            if (tex != null) 
               backgroundApp.setTexture(tex.getTexture());
         }      

     
         // Load the grass texture on the ground
         if (floorTexture) {
            // Create the ground, a textured XZ plane
            Shape3D groundPlane = null;
            groundPlane = new TexturedXYPlane("images/natfl014_3.jpg",200.0f,200.0f);
            Transform3D rotate = new Transform3D();
            rotate.rotX(-Math.PI/2.0f);
            Transform3D tg = new Transform3D();
            tg.set(new Vector3f(0.0f, -0.15f, 0.0f));
            TransformGroup trl = new TransformGroup(tg);
            TransformGroup rt = new TransformGroup(rotate);
            trl.addChild(rt);
            rt.addChild(groundPlane);
            objScale.addChild(trl);
             
         } else {
            // Create the ground, a Box object
            Appearance groundAppear = new Appearance();
            Color3f eColor    = new Color3f(0.0f, 0.0f, 0.0f);
            Color3f sColor    = new Color3f(1.0f, 1.0f, 1.0f);
            Color3f oColor    = new Color3f(0.5f, 1.0f, 0.35f);
            Box ground = new Box(200,0.1f,200,Box.GENERATE_NORMALS, groundAppear);
            Transform3D tg = new Transform3D();
            tg.set(new Vector3f(0.0f, -0.15f, 0.0f));
            TransformGroup trl = new TransformGroup(tg);
            trl.addChild(ground);
            Material groundMat = new Material(oColor, eColor, oColor, sColor, 100.0f);
            groundMat.setLightingEnable(true);
            groundAppear.setMaterial(groundMat);
            objScale.addChild(trl);
        }      
      }
      
      // Shine it with two lights.
      Color3f lColor1 = new Color3f(0.7f, 0.7f, 0.7f);
      Color3f lColor2 = new Color3f(0.2f, 0.2f, 0.1f);
      Vector3f lDir2  = new Vector3f(1.0f, -1.0f, 1.0f);
      Vector3f lDir1  = new Vector3f(-1.0f, -1.0f, -1.0f);
      DirectionalLight lgt1 = new DirectionalLight(lColor1, lDir1);
      DirectionalLight lgt2 = new DirectionalLight(lColor2, lDir2);
      lgt1.setInfluencingBounds(bounds);
      lgt2.setInfluencingBounds(bounds);
      objScale.addChild(lgt1);
      objScale.addChild(lgt2);
      
      // Behavior nodes for the mouse and keyboard.
        
      // Create the rotate behavior node
      MouseRotate behavior1 = new MouseRotate(viewTrans);
      objRoot.addChild(behavior1);
      behavior1.setSchedulingBounds(bounds);

      // Create the zoom behavior node
      MouseZoom behavior2 = new MouseZoom(viewTrans);
      objRoot.addChild(behavior2);
      behavior2.setSchedulingBounds(bounds);

      // Create the translate behavior node
      MouseTranslate behavior3 = new MouseTranslate(viewTrans);
      objRoot.addChild(behavior3);
      behavior3.setSchedulingBounds(bounds);
      
      // Create the key navigation behavior node
      KeyNavigatorBehavior keyNavBeh = new KeyNavigatorBehavior(viewTrans);
      keyNavBeh.setSchedulingBounds(new BoundingSphere(new Point3d(),1000.0));
      objRoot.addChild(keyNavBeh);
   }
   
   // Adds an Accident Object (e.g. a road, a roadsign etc.)
   public void add(AccidentObject object) {
      ainfo.addStaticObject(object);
   }
   
   // Adds a Dynamic Accident Object (e.g. a vehicle) 
   public void add(DynamicAccidentObject object) {
      ainfo.addDynamicObject(object);
   }
   
   // Returns the scenegraph
   public BranchGroup getSceneGraph() {
      return objRoot;
   }
   
   // Finalize the AccidentUniverse
   public void createAll() {
      createGeometry();
      // Create the geometry of the static objects
      for (int i=0; i<ainfo.getStaticAmount(); i++) {
         ainfo.getStaticObject(i).createGeometry();
         objScale.addChild(ainfo.getStaticObject(i).getGeometry());
      }
      // Create the geometry of the dynamic objects
      for (int i=0; i<ainfo.getDynamicAmount(); i++) {
         ainfo.getDynamicObject(i).createGeometry();
         objScale.addChild(ainfo.getDynamicObject(i).getGeometry());
      }
      objRoot.compile();
      u.addBranchGraph(objRoot);
   }
   
   // Returns the canvas for viewing
   public Canvas3D getCanvas() {
      return (Canvas3D)c;
   }
   
   // Returns the accident info object, containing all dynamic and static information
   public AccidentInfo getAccidentInfo() {
      return ainfo;
   }
   
   // Dynamic controls
   
   // Start the simulation
   public void play() {
      for (int i = 0; i<ainfo.getDynamicAmount(); i++)
         ainfo.getDynamicObject(i).play();
   }
   
   // Pause the simulation
   public void pause() {
      for (int i = 0; i<ainfo.getDynamicAmount(); i++)
         ainfo.getDynamicObject(i).pause();
   }
   
   // Resume the simulation
   public void resume() {
      for (int i = 0; i<ainfo.getDynamicAmount(); i++)
         ainfo.getDynamicObject(i).resume();
   }
   
   // Stop the simulation
   public void stop() {
      for (int i = 0; i<ainfo.getDynamicAmount(); i++)
         ainfo.getDynamicObject(i).stop();
   }   
}
