// This class defines an object containing the information necessary to model
// a collision

// 	Name: CollisionInfo (class)
//	Author: Arjan Egges
//	Version: 1.0

//	Version 1.1 by Bastian Schulz
//	New since Version 1.1: 
//	public float collCoordX renamed to public float xPos
//	resp.
//	public float collCoordY renamed to public float yPos
//	in order to stay conform with other objects that use the XML tag <coords>

package  se.lth.cs.carsimgui;

class CollisionInfo {

   // The index of the actor in the dynamic object array
   public int actorIndex;
   
   // The index of the victim in the dynamic/static (!) object array
   public int victimIndex;
   
   // Variable used to define if the victim is a dynamic or a static object
   public int victimKind;
   
   // The vehicle part of the actor in the collision
   public int actorPart;

   // The vehicle part of the victim in the collision
   public int victimPart;
   
   // Determines, if the actor of the collision should overturn (to the left or right side) or not
   public int actorOverturn;

   // Determines, if the vistim of the collision should overturn (to the left or right side) or not
   public int victimOverturn;
   
   // Collision coordinates
   public float xPos = 0.0f;
   public float yPos = 0.0f;
   
   // Constructor
   public CollisionInfo() {
      actorIndex = -1;
      victimIndex = -1;
      victimKind = Constant.DYNAMIC_OBJECT;
      actorPart = Constant.UNKNOWN_PART;
      victimPart = Constant.UNKNOWN_PART;
      actorOverturn = Constant.NO_OVERTURN;
      victimOverturn = Constant.NO_OVERTURN;
   }
}