// This class provides constants used in the AccidentUniverse modeling.

// 	Name: Constant (class)
//	Author: Arjan Egges
//	Version: 1.0

package  se.lth.cs.carsimgui;


class Constant {

   // Constants used for victim description in the CollisionInfo
   static final int STATIC_OBJECT = 0;
   static final int DYNAMIC_OBJECT = 1;
   

   // Constants used by the Road class and the Event class and for overturns
   static final int STRAIGHTROAD 	= 0;   // looks like	|
   static final int CROSSROAD 		= 1;	// looks like	-|-
   static final int ROADSLEFT 		= 2;	// looks like 	-|
   static final int ROADSRIGHT 		= 3;	// looks like	|-
   static final int TURNLEFT 		= 4;
   static final int TURNRIGHT 		= 5;
   static final int DEPASS 		= 6;
   static final int STOP 		= 7;
   static final int CHANGELANE_LEFT 	= 8;
   static final int CHANGELANE_RIGHT 	= 9;
   static final int LEAVEROAD 		= 10;
   static final int LEAVEROAD_LEFT 	= 11;
   static final int LEAVEROAD_RIGHT 	= 12;
   static final int OVERTURN		= 13;
   static final int OVERTURN_LEFT	= 14;
   static final int OVERTURN_RIGHT	= 15;
   static final int NO_OVERTURN		= 16;

   // Orientation
   static final int NORTH 		= 1;
   static final int EAST 		= 2;
   static final int SOUTH 		= 3;
   static final int WEST 		= 4;
   
   // Colors for the traffic light
   static final int RED 		= 0;
   static final int GREEN 		= 1;
   static final int ORANGE 		= 2;
   static final int INACTIVE 		= 3;
   
   // Vehicle parts
   static final int FRONT 		= 0;
   static final int REAR 		= 1;
   static final int LEFTSIDE 		= 2;
   static final int RIGHTSIDE 		= 3;
   static final int UNKNOWN_PART 	= 4;
       
   // Scenario constants used by the Accident planner
   static final int SCENARIO_I 		= 0;
   static final int SCENARIO_II 	= 1;
   static final int SCENARIO_III 	= 2;
   static final int SCENARIO_IV 	= 3;
   static final int SCENARIO_V 		= 4;
   static final int SCENARIO_VI 	= 5;
   
   static final int UNKNOWN_SCENARIO = 999;   
   
   // Returns a string representation of a scenario
   static String scenarioToString(int scenario) {
      if (scenario == SCENARIO_I)
         return "scenario_I (turn left)";
      else if (scenario == SCENARIO_II)
         return "scenario_II (turn right)";
      else if (scenario == SCENARIO_III)
         return "scenario_III (depass)";
      else if (scenario == SCENARIO_IV)
         return "scenario_IV (change lane left)";
      else if (scenario == SCENARIO_V)
         return "scenario_V (change lane right)";
      else if (scenario == SCENARIO_VI)
         return "scenario_VI (straight)";
      else
         return "unknown_scenario";
   }
   
   // Returns a string representation of the event kind
   static String kindToString(int kind) {
      if (kind == STRAIGHTROAD) 
         return " driving forward";
      else if (kind == TURNLEFT)
         return " turning left";
      else if (kind == TURNRIGHT)
         return " turning right";
      else if (kind == DEPASS)
         return " depassing";
      else if (kind == STOP)
         return " stopping";
      else if (kind == CHANGELANE_LEFT)
         return " changing lane to the left";
      else if (kind == CHANGELANE_RIGHT)
         return " changing lane to the right";
      else if (kind == LEAVEROAD)
         return " leaving road";
      else if (kind == LEAVEROAD_LEFT)
         return " leaving road to the left";
      else if (kind == LEAVEROAD_RIGHT)
         return " leaving road to the right";
      else if (kind == OVERTURN)
         return " overturn";
      else if (kind == OVERTURN_LEFT)
         return " overturn to the left side";
      else if (kind == OVERTURN_RIGHT)
         return " overturn to the right side";
      else
         return " unknown_action";
   }
   
   // Returns a string representations of a vehicle part
   static String partToString(int part) {
      if (part == FRONT)
         return "front";
      else if (part == REAR)
         return "rear";
      else if (part == LEFTSIDE)
         return "left side";
      else if (part == RIGHTSIDE)
         return "right side";
      else 
         return "unknown part";
   }

   // Returns a string representations of an orientation
   static String orientationToString(int orientation) {
      if (orientation == NORTH)
         return "north";
      else if (orientation == EAST)
         return "east";
      else if (orientation == SOUTH)
         return "south";
      else if (orientation == WEST)
         return "west";
      else 
         return "unknown orientation";
   }

}