// Frame that generates a demo of a complex trafic situation

// 	Name: DemoTwo (class)
//	Author: Arjan Egges
//	Version: 1.0

package  se.lth.cs.carsimgui;


import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.sun.j3d.utils.applet.MainFrame;



class DemoTwo extends Frame implements ActionListener, WindowListener {
    // Window Components
    Panel controlPanel;
    Panel canvasPanel;
    Button playpauseButton;
    Button stopButton;
                   
    AccidentUniverse au;
    
    // Variables for handling dynamics
    boolean playing = false;
    boolean playFromBegin = true;
    
  public DemoTwo(){
  
      super("Demonstration [idle]");
      addWindowListener(this);
      
      // Introduction message
      Messenger.message("This demonstration shows an animation of a complex trafic situation.\n");
      Messenger.message("Vehicle 1 arrives from the west (left) road of the crossing and has to\n");
      Messenger.message("wait for vehicle 2 that arrives from the south and wants to turn left.\n");
      Messenger.message("Press play to run the demonstration\n");

      au = new AccidentUniverse();

      // Create a crossroad.
      Road crossRoad = new Road();
      crossRoad.type = Constant.CROSSROAD;
      au.add(crossRoad);
      
      // Create two cars and their corresponding eventchains
      FrenchDuck vehicle1 = new FrenchDuck();
      EventChain v1EventChain = new EventChain();
      FrenchDuck vehicle2 = new FrenchDuck();
      EventChain v2EventChain = new EventChain();
      
      // Create the event chain of vehicle 1. This event chain consists of 
      // three parts:

      // 1. Arrival at the crossroad
      Event v1Event1 = new Event();
      v1Event1.name = "vehicle 1 arrives at crossroad";
      v1Event1.kind = Constant.STRAIGHTROAD;
      v1Event1.startPosX = -30.0f;
      v1Event1.startPosY = 1.5f;
      v1Event1.endPosX = -5.0f;
      v1Event1.endPosY = 1.5f;
      v1EventChain.addEvent(v1Event1);

      // 2. Stop and wait for vehicle 2, which is coming from the right
      Event v1Event2 = new Event();
      v1Event2.name = "vehicle 1 waits for vehicle 2";
      v1Event2.kind = Constant.STOP;
      v1EventChain.addEvent(v1Event2);

      // 3. Continue driving
      Event v1Event3 = new Event();
      v1Event3.name = "vehicle 1 continues driving";
      v1Event3.kind = Constant.STRAIGHTROAD;
      v1Event3.startPosX = -5.0f;
      v1Event3.startPosY = 1.5f;
      v1Event3.endPosX = 30.0f;
      v1Event3.endPosY = 1.5f;
      v1EventChain.addEvent(v1Event3); 
      
      // Create the event chain of vehicle 2. This event chain consists of 
      // three event: driving forward, a turn, and again driving forward
      Event v2Event1 = new Event();
      v2Event1.name = "vehicle 2 arrives at crossroad";
      v2Event1.startPosX = 1.5f;
      v2Event1.startPosY = 20.0f;
      v2Event1.endPosX = 1.5f;
      v2Event1.endPosY = 6.0f;
      v2EventChain.addEvent(v2Event1);
      
      Event v2Event2 = new Event();
      v2Event2.name = "vehicle 2 turns left";
      v2Event2.kind = Constant.TURNLEFT;
      v2Event2.startPosX = 1.5f;
      v2Event2.startPosY = 6.0f;
      v2Event2.endPosX = -6.0f;
      v2Event2.endPosY = -1.5f;
      v2EventChain.addEvent(v2Event2);

      Event v2Event3 = new Event();
      v2Event3.name = "vehicle 2 driving forward";
      v2Event3.startPosX = -6.0f;
      v2Event3.startPosY = -1.5f;
      v2Event3.endPosX = -20.0f;
      v2Event3.endPosY = -1.5f;
      v2EventChain.addEvent(v2Event3);

      // Create the trajectories
      TrajectoryPlanner tp1 = new TrajectoryPlanner(v1EventChain);
      tp1.plan();
      vehicle1.setTrajectory(tp1.getTrajectory());
      
      TrajectoryPlanner tp2 = new TrajectoryPlanner(v2EventChain);
      tp2.plan();
      vehicle2.setTrajectory(tp2.getTrajectory());
      
      
      // Add both vehicles to the universe.
      au.add(vehicle1);
      au.add(vehicle2);      

      // Create the temporal values
      TemporalPlanner tp = new TemporalPlanner(au.getAccidentInfo());
      tp.stopPerc = 30.0f;
      tp.plan();

      // Finalize
      au.createAll();
            
      // Create the canvas and the control panel
      this.setLayout(new FlowLayout()); 
      this.setSize(600,650);  
      canvasPanel = new Panel();
      controlPanel = new Panel();
      createControlPanel(controlPanel); 
      this.add(controlPanel);
      canvasPanel.add(au.getCanvas());
      this.add(canvasPanel);
      this.show();
   }
    
    private void createControlPanel(Panel p) {
       stopButton = new Button("Stop");
       playpauseButton = new Button("Play");
              
       p.add(stopButton);
       p.add(playpauseButton);
               
       stopButton.addActionListener (this);
       playpauseButton.addActionListener (this);
    }    

    // Button event handling
    public void actionPerformed (ActionEvent event) {
      if (event.getSource() == playpauseButton) {
          if (!playing) {
            setTitle("Demonstration [running]");
            playing = true;
            playpauseButton.setLabel("Pause");
            if (playFromBegin)
               au.play();
            else
               au.resume();
          } else {
             setTitle("Demonstration [paused]");
             au.pause();
             playFromBegin = false;
             playing = false;
             playpauseButton.setLabel("Play");
          }
      } else if (event.getSource() == stopButton) {
         setTitle("Demonstration [idle]");
         au.stop();
         playing = false;
         playFromBegin = true;
         playpauseButton.setLabel("Play");
      }  
    }   
    
    // Additional methods for  window event handling.
    
    public void windowClosing( WindowEvent event ) {
       Messenger.message("End of demonstration...\n");
       dispose();
    }
   
    public void windowClosed( WindowEvent event ) {
    }
   
    public void windowDeiconified( WindowEvent event ) {}
   
    public void windowIconified( WindowEvent event ) {}
   
   
public void windowDeactivated( WindowEvent event ) {}
   
    public void windowActivated( WindowEvent event ) {}
   
    public void windowOpened( WindowEvent event ) {}
}


   