// This class checks if the delivered XML document is valid against the accident.dtd 

// 	Name: FormalismChecker (class)
//	Version: 1.1 by Bastian Schulz

package  se.lth.cs.carsimgui;


import java.io.*;
import javax.xml.parsers.ParserConfigurationException;  
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;      
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;      
import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException;

class FormalismChecker extends DefaultHandler {

	private boolean DocumentIsValid = true;
	private String tempStr;
	private int criticalCount;
	

	public FormalismChecker() {
		super();
	}

	public void startDocument() throws org.xml.sax.SAXException
	{
		DocumentIsValid = true;
	}

	public void endDocument() throws org.xml.sax.SAXException {
	
	}

	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
	{
		if (qName.equals("eventChain"))
		{
			criticalCount = 0;
		}
		else if (qName.equals("event"))
		{
			tempStr = attributes.getValue("critical");
			if (tempStr.equals("yes"))
			{
				criticalCount++;
			}
		}
	}

	public void endElement(String uri, String localName, String qName) throws SAXException
	{
		if (qName.equals("eventChain"))
		{
			if (criticalCount > 1) {
				Messenger.message("Only 1 critical event allowed in an event chain\n");
				DocumentIsValid = false;
			}
		}
	}

	public void characters(char[] ch, int start, int length) throws SAXException{
	
	}

	public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
	
	}

	public void error(SAXParseException ex) throws SAXException {
		File fInput = new File (ex.getSystemId());
		Messenger.message("Error in " + fInput.getName() + " at line " + ex.getLineNumber() + ".\nDescription: " + ex + "\nPath:        " + fInput.getPath() );
		DocumentIsValid = false;
	}

	public void fatalError(SAXParseException ex) throws SAXException {
		File fInput = new File (ex.getSystemId());
		Messenger.message("Fatal error in " + fInput.getName() + " at line " + ex.getLineNumber() + ".\nDescription: " + ex);
		DocumentIsValid = false;
	}

	public void processingInstruction(String arg1, String arg2) throws SAXException {
	}

	public void setDocumentLocator(org.xml.sax.Locator arg1) {
	}

	public void warning(SAXParseException ex) throws SAXException {
		File fInput = new File(ex.getSystemId());
		System.err.println("Warning: " + fInput.getName() + ": at line " + ex.getLineNumber() + ".\nDescription: " + ex);
	}

	
	
	public boolean parse(String parseString)
	{
		try {
			// Write the String to the file xmltemp
			FileOutputStream fileout = new FileOutputStream("xmltemp");
			DataOutputStream dataout = new DataOutputStream(fileout);
			dataout.writeBytes(parseString);
			dataout.flush();
			dataout.close();
			parseFile(true,this,"xmltemp");
		} catch (Exception e) {
			System.err.println("Error: " + e);
		}
		return DocumentIsValid;
	}

	protected void parseFile(boolean isValidating, DefaultHandler handler, String sFilename)
	{
		try {
			// Get a "parser factory", an object that creates parsers
	
		SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
	
		// Set up the factory to create the appropriate type of parser
			saxParserFactory.setValidating(isValidating);
	
		saxParserFactory.setNamespaceAware(false); 
	
		
			SAXParser parser = saxParserFactory.newSAXParser();
	
		
			parser.parse(new File(sFilename), handler);
		} catch (Exception ex) {
			Messenger.message("Exception: " + ex);
		}
	}
}