// This class parses the accident formalism and generates a simple accident universe that 
// contains all the objects recognised (without planning!)
// In Version 1.1 the class uses the XML syntax and validates the XML data 

// 	Name: FormalismParser (class)
//	Version: 1.0 by Arjan Egges
//	Version: 1.1 by Bastian Schulz

package  se.lth.cs.carsimgui;


import java.io.*;
import javax.xml.parsers.ParserConfigurationException;  
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;      
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;      
import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException;

class FormalismParser extends DefaultHandler {

	private boolean DocumentIsValid = true;
	private AccidentUniverse au;
	private Object tempObject = new Object();
	private String tempStr;
	private int criticalCount;
	private String thisPackage = this.getClass().getPackage().getName();
	
	// Variables used by the Accident Planner
	AccidentInfo ainfo;

	public FormalismParser() {
		super();
		// Initialise the accidentUniverse
		au = new AccidentUniverse();
	}

	public void startDocument() throws org.xml.sax.SAXException {

	}

	public void endDocument() throws org.xml.sax.SAXException {
	
	}

	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
		
		// to debug the class, in which part of the xml doc an error occurs
		// System.out.println("<"+qName+">");
		
		if (qName.equals("road"))
		{
			Road road = new Road();
			tempStr = attributes.getValue("kind");
			if (tempStr.equals("crossroad"))
			{
				road.type = Constant.CROSSROAD;
			}
			else if (tempStr.equals("turn_left"))
			{
				road.type = Constant.TURNLEFT;
			}
			else if (tempStr.equals("turn_right"))
			{
				road.type = Constant.TURNRIGHT;
			}
			else if (tempStr.equals("straightroad"))
			{
				// do nothing
			}
			tempObject = road;
		}
		else if (qName.equals("tree"))
		{
			Tree tree = new Tree();
			tree.id = attributes.getValue("id");
			tempObject = tree;
		}
		else if (qName.equals("sign"))
		{
			StopSign ss = new StopSign();
			// The next line must be activated if there are several kinds of signs
			//tempStr = attributes.getValue("kind");
			tempObject = ss;
		}
		else if (qName.equals("trafficLight"))
		{
			TrafficLight tl = new TrafficLight();
			tl.id = attributes.getValue("id");
			tempStr = attributes.getValue("colorType");
			if (tempStr.equals("red")) 
				tl.color = Constant.RED;
			else if (tempStr.equals("orange")) 
				tl.color = Constant.ORANGE;
			else if (tempStr.equals("green")) 
				tl.color = Constant.GREEN;
			else if (tempStr.equals("inactive")) 
				tl.color = Constant.INACTIVE;      
			tempObject = tl;
		}
		else if (qName.equals("levelCrossing"))
		{
			LevelCrossing lc = new LevelCrossing();
			tempObject = lc;
		}
		else if (qName.equals("vehicle"))
		{
			DynamicAccidentObject dynAccObj = new SimpleCar();
			tempStr = attributes.getValue("kind");
			if (tempStr.equals("car")) 
			{
				dynAccObj = new FrenchDuck();
			}
			else if (tempStr.equals("truck")) 
			{
				((SimpleCar)dynAccObj).length = 6.0f;
				((SimpleCar)dynAccObj).height = 2.0f;
			}
			dynAccObj.setId(attributes.getValue("id"));
      			
      			
			// All information of the inner tags is saved in a DynamicInfo object
			// e.g. initDirection, startSign, endSign, event
			
			DynamicInfo currDynInfo = new DynamicInfo();
      			tempStr = attributes.getValue("initDirection");
			if (tempStr.equals("north")) 
			{
				currDynInfo.initDirection = DynamicInfo.NORTH;
			}
			else if (tempStr.equals("east")) 
			{
				currDynInfo.initDirection = DynamicInfo.EAST;
			}
			else if (tempStr.equals("south")) 
			{
				currDynInfo.initDirection = DynamicInfo.SOUTH;
			}
			else if (tempStr.equals("west")) 
			{
				currDynInfo.initDirection = DynamicInfo.WEST;
			}
			
			tempObject = currDynInfo;

			// The dynamicAccidentObject can already added to the AccidentUnivesum
      			au.add(dynAccObj);
		}
		else if (qName.equals("startSign"))
		{
			// do nothing
		}
		else if (qName.equals("endSign"))
		{
			// do nothing
		}
		else if (qName.equals("eventChain"))
		{
			criticalCount = 0;
		}
		else if (qName.equals("event"))
		{
			Event e = new Event();
			
			tempStr = attributes.getValue("critical");
			if (tempStr.equals("yes"))
			{
				e.critical = true;
				criticalCount++;
			}
			
			tempStr = attributes.getValue("kind");
			if (tempStr.equals("driving_forward"))
				e.kind = Constant.STRAIGHTROAD;
			else if (tempStr.equals("turn_left"))
				e.kind = Constant.TURNLEFT;
			else if (tempStr.equals("turn_right"))
				e.kind = Constant.TURNRIGHT;
			else if (tempStr.equals("stop")) 
				e.kind = Constant.STOP;
			else if (tempStr.equals("depass"))
				e.kind = Constant.DEPASS;
			else if (tempStr.equals("change_lane_left"))
				e.kind = Constant.CHANGELANE_LEFT;
			else if (tempStr.equals("change_lane_right"))
				e.kind = Constant.CHANGELANE_RIGHT;
			else if (tempStr.equals("leave_road"))
				e.kind = Constant.LEAVEROAD;
			else if (tempStr.equals("leave_road_left"))
				e.kind = Constant.LEAVEROAD_LEFT;
			else if (tempStr.equals("leave_road_right"))
				e.kind = Constant.LEAVEROAD_RIGHT;
			else if (tempStr.equals("overturn"))
				e.kind = Constant.OVERTURN;
			else if (tempStr.equals("overturn_left"))
				e.kind = Constant.OVERTURN_LEFT;
			else if (tempStr.equals("overturn_right"))
				e.kind = Constant.OVERTURN_RIGHT;

			((DynamicInfo)tempObject).ceb.addEvent(e);
		}
		else if (qName.equals("collision"))
		{
			CollisionInfo ci = new CollisionInfo();
			tempObject = ci;
		}
		else if (qName.equals("actor"))
		{
			tempStr = attributes.getValue("id");
			// Search the corresponding index
			boolean found = false;
			int i=0;
			while (!found && i<ainfo.getDynamicAmount())
			{
				if (tempStr.equals(ainfo.getDynamicObject(i).getId()))
 					found = true;
				else
					i++;
			}
			if (found)
			{
				((CollisionInfo)tempObject).actorIndex = i;
			}
			else
			{
				Messenger.message("Error: vehiclename <"+tempStr+"> not found.\n");
				DocumentIsValid = false;
			}
			
			
			tempStr = attributes.getValue("side");
			if (tempStr.equals("front")) {
				((CollisionInfo)tempObject).actorPart = Constant.FRONT;
			} else if (tempStr.equals("rear")) {
				((CollisionInfo)tempObject).actorPart = Constant.REAR;
			} else if (tempStr.equals("leftside")) {
				((CollisionInfo)tempObject).actorPart = Constant.LEFTSIDE;
			} else if (tempStr.equals("rightside")) {
				((CollisionInfo)tempObject).actorPart = Constant.RIGHTSIDE;
			} else if (tempStr.equals("unknown")) {
				((CollisionInfo)tempObject).actorPart = Constant.UNKNOWN_PART;
			}


			tempStr = attributes.getValue("overturn");
			if (tempStr.equals("no")) {
				((CollisionInfo)tempObject).actorOverturn = Constant.NO_OVERTURN;
			} else if (tempStr.equals("unknown")) {
				((CollisionInfo)tempObject).actorOverturn = Constant.OVERTURN;
			} else if (tempStr.equals("left")) {
				((CollisionInfo)tempObject).actorOverturn = Constant.OVERTURN_LEFT;
			} else if (tempStr.equals("right")) {
				((CollisionInfo)tempObject).actorOverturn = Constant.OVERTURN_RIGHT;
			}
		}
		else if (qName.equals("victim"))
		{
			tempStr = attributes.getValue("id");
			// Search the corresponding index
			boolean found = false;
			int i=0;
			while (!found && i<ainfo.getDynamicAmount())
			{
				if (tempStr.equals(ainfo.getDynamicObject(i).getId()))
 					found = true;
				else
					i++;
			}
			// The victim can be a static object as well
			if (!found)
			{
				i = 0;
				while (!found && i<ainfo.getStaticAmount())
				{
					if (tempStr.equals(ainfo.getStaticObject(i).getId()))
					{
						found = true;
						((CollisionInfo)tempObject).victimKind = Constant.STATIC_OBJECT;
					}
					else
						i++;
				}
			}
			
			if (found)
			{
				((CollisionInfo)tempObject).victimIndex = i;
			}
			else
			{
				Messenger.message("Error: vehiclename <"+tempStr+"> not found.\n");
				DocumentIsValid = false;
			}
			
			
			tempStr = attributes.getValue("side");
			if (tempStr.equals("front")) {
				((CollisionInfo)tempObject).victimPart = Constant.FRONT;
			} else if (tempStr.equals("rear")) {
				((CollisionInfo)tempObject).victimPart = Constant.REAR;
			} else if (tempStr.equals("leftside")) {
				((CollisionInfo)tempObject).victimPart = Constant.LEFTSIDE;
			} else if (tempStr.equals("rightside")) {
				((CollisionInfo)tempObject).victimPart = Constant.RIGHTSIDE;
			} else if (tempStr.equals("unknown")) {
				((CollisionInfo)tempObject).victimPart = Constant.UNKNOWN_PART;
			}


			tempStr = attributes.getValue("overturn");
			if (tempStr.equals("no")) {
				((CollisionInfo)tempObject).victimOverturn = Constant.NO_OVERTURN;
			} else if (tempStr.equals("unknown")) {
				((CollisionInfo)tempObject).victimOverturn = Constant.OVERTURN;
			} else if (tempStr.equals("left")) {
				((CollisionInfo)tempObject).victimOverturn = Constant.OVERTURN_LEFT;
			} else if (tempStr.equals("right")) {
				((CollisionInfo)tempObject).victimOverturn = Constant.OVERTURN_RIGHT;
			}
		}
		else if (qName.equals("coords"))
		{
			// Independent from the content in tempObject.
			// Due to the DTD it can be an object of the following classes:
			// Tree, Sign, TrafficLight, LevelCrossing, CollisionInfo
			// To avoid type errors the type of tempObjects has to be known
			
			tempStr = tempObject.getClass().getName();
			if (tempStr.equals(thisPackage+".Tree"))
			{
				((Tree)tempObject).xPos = Float.parseFloat(attributes.getValue("x"));
				((Tree)tempObject).yPos = Float.parseFloat(attributes.getValue("y"));
			} 
			else if (tempStr.equals(thisPackage+".StopSign"))
			{
				((StopSign)tempObject).xPos = Float.parseFloat(attributes.getValue("x"));
				((StopSign)tempObject).yPos = Float.parseFloat(attributes.getValue("y"));
			}
			else if (tempStr.equals(thisPackage+".TrafficLight"))
			{
				((TrafficLight)tempObject).xPos = Float.parseFloat(attributes.getValue("x"));
				((TrafficLight)tempObject).yPos = Float.parseFloat(attributes.getValue("y"));
			}
			else if (tempStr.equals(thisPackage+".LevelCrossing"))
			{
				((LevelCrossing)tempObject).xPos = Float.parseFloat(attributes.getValue("x"));
				((LevelCrossing)tempObject).yPos = Float.parseFloat(attributes.getValue("y"));
			}
			else if (tempStr.equals(thisPackage+".CollisionInfo"))
			{
				((CollisionInfo)tempObject).xPos = Float.parseFloat(attributes.getValue("x"));
				((CollisionInfo)tempObject).yPos = Float.parseFloat(attributes.getValue("y"));
			}
		}
	}

	public void endElement(String uri, String localName, String qName) throws SAXException {
		
		// to debug the class, in which part of the xml doc an error occurs
		// System.out.println("</"+qName+">");
		
		if (qName.equals("road"))
		{
			au.add((Road)tempObject);
		}
		else if (qName.equals("tree"))
		{
			au.add((Tree)tempObject);
		}
		else if (qName.equals("sign"))
		{
			au.add((StopSign)tempObject);
		}
		else if (qName.equals("trafficLight"))
		{
			au.add((TrafficLight)tempObject);
		}
		else if (qName.equals("levelCrossing"))
		{
			au.add((LevelCrossing)tempObject);
		}
		else if (qName.equals("vehicle"))
		{
			ainfo.addDynamicInfo((DynamicInfo)tempObject);
		}
		else if (qName.equals("startSign"))
		{
			// if the xml doc contains a </startSign> element (it's optional), 
			// tempStr contains now the name of the street.
			
			Roadsign rs = new Roadsign(tempStr);
			rs.xPos = -10.0f;
			au.add(rs);
			((DynamicInfo)tempObject).startsignIndex = ainfo.getStaticAmount()-1;
		}
		else if (qName.equals("endSign"))
		{
			// if the xml doc contains a </endSign> element (it's optional), 
			// tempStr contains now the name of the street.
			
			Roadsign rs = new Roadsign(tempStr);
			rs.xPos = 10.0f;
			au.add(rs);
			((DynamicInfo)tempObject).endsignIndex = ainfo.getStaticAmount()-1;
		}
		else if (qName.equals("eventChain"))
		{
			if (criticalCount > 1) {
				Messenger.message("Only 1 critical event allowed in an event chain\n");
				DocumentIsValid = false;
			}
		}
		else if (qName.equals("event"))
		{
			// do nothing
		}
		else if (qName.equals("collision"))
		{
			ainfo.addCollisionInfo(((CollisionInfo)tempObject));
		}
		else if (qName.equals("actor"))
		{
			// do nothing
		}
		else if (qName.equals("victim"))
		{
			// do nothing
		}
		else if (qName.equals("coords"))
		{
			// do nothing
		}
	}

	public void characters(char[] ch, int start, int length) throws SAXException
	{
		// to safe the #PCDATA of startSign end endSign
		String s = new String(ch,start,length);
		tempStr = s.trim();
	}

	public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
	
	}

	public void error(SAXParseException ex) throws SAXException {
		File fInput = new File (ex.getSystemId());
		System.err.println("Error in " + fInput.getName() + " at line " + ex.getLineNumber() + ".\nDescription: " + ex + "\nPath:        " + fInput.getPath() );
		DocumentIsValid = false;
	}

	public void fatalError(SAXParseException ex) throws SAXException {
		File fInput = new File (ex.getSystemId());
		System.err.println("Error in " + fInput.getName() + " at line " + ex.getLineNumber() + ".\nDescription: " + ex);
		DocumentIsValid = false;
	}

	public void processingInstruction(String arg1, String arg2) throws SAXException {
	}

	public void setDocumentLocator(org.xml.sax.Locator arg1) {
	}

	public void warning(SAXParseException ex) throws SAXException {
		File fInput = new File(ex.getSystemId());
		System.err.println("Warning: " + fInput.getName() + ": at line " + ex.getLineNumber() + ".\nDescription: " + ex);
	}

	
	
	public void parse(String parseString)
	{
		try {
			// Write the String to the file xmltemp
			FileOutputStream fileout = new FileOutputStream("xmltemp");
			DataOutputStream dataout = new DataOutputStream(fileout);
			dataout.writeBytes(parseString);
			dataout.flush();
			dataout.close();
			parseFile(true,this,"xmltemp");
		} catch (Exception e) {
			System.err.println("Error: " + e);
		}
	}

	protected void parseFile(boolean isValidating, DefaultHandler handler, String sFilename) {
		try
		{
			// Get a "parser factory", an an object that creates parsers
	
		ainfo = au.getAccidentInfo();
			SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
	
		// Set up the factory to create the appropriate type of parser
			saxParserFactory.setValidating(isValidating);
	
		saxParserFactory.setNamespaceAware(false); 
	
		
			SAXParser parser = saxParserFactory.newSAXParser();
	
		
			parser.parse(new File(sFilename), handler);
	
		// Messenger.message("Number of static objects: "+String.valueOf(ainfo.getStaticAmount())+"\n");
			// Messenger.message("Number of dynamic objects: "+String.valueOf(ainfo.getDynamicAmount())+"\n");

		}
		catch (Exception ex)
		{
			System.err.println("Exception: " + ex);
			System.exit(2);
		}
	}

	public AccidentUniverse getUniverse() {
		return au;
	}
   
	public boolean errors() {
		return (!DocumentIsValid);
	}

}