// This class reads the VRML file "2CVV3.wrl" which contains a VRML model of
// the 2CV, created by Fabrice TABORDET.

// 	Name: FrenchDuck (class)
//	Author: Arjan Egges
//	Version: 1.0

package  se.lth.cs.carsimgui;


import com.sun.j3d.utils.behaviors.interpolators.*;
import java.util.Date;
import javax.media.j3d.*;
import javax.vecmath.*;

class FrenchDuck implements DynamicAccidentObject {
   
   private String id = "";
   
   private TransformGroup movingCar;
   private VrmlGroupLoader vgl;

   // Private members for dynamics
   private RotPosScaleTCBSplinePathInterpolator rotPosInt;
   private Trajectory traject;
   private Alpha alpha;
   private long timeStamp;
   private long startTime;  

   // Constructor
   public FrenchDuck() {
      traject = new Trajectory();
      timeStamp = new Date().getTime(); 
   }
   
   // Creates the geometry of the French Duck
   public void createGeometry() {
      // Dynamic property variables
      float startPosX = traject.get(0).x;
      float startPosY = traject.get(0).y;

      // Load the Vrml-file and set to the right scaling
      vgl = new VrmlGroupLoader("2CVV3.wrl");
      Transform3D scaling = new Transform3D();
      scaling.setScale(0.3);
      TransformGroup fDuck = vgl.getTransformGroup();
      fDuck.setTransform(scaling);
      
      Transform3D translate = new Transform3D();
      Transform3D rotate = new Transform3D();
      translate.set(new Vector3f(startPosX, 0.0f, startPosY));
      rotate.rotY(traject.getInitAngle());
      translate.mul(rotate);
      movingCar = new TransformGroup(translate);
      
      movingCar.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
      
      // Create behavior
      TrajectoryBehavior tb = new TrajectoryBehavior(this);
      tb.createBehavior();
      alpha = tb.alpha;
      
      rotPosInt = tb.getBehavior();
      rotPosInt.setEnable(false);
      movingCar.addChild(fDuck);
      
      movingCar.addChild(rotPosInt);
   }
   
   // Returns the geometry of the French Duck
   public TransformGroup getGeometry() {
      return movingCar;
   }

   // Sets the trajectory of the FrenchDuck object
   public void setTrajectory(Trajectory t) {
      traject = t;
   }
   
   // Returns the trajectory of the FrenchDuck object
   public Trajectory getTrajectory() {
      return traject;
   }
   
   // Some methods for handling dynamics 
   
   // Start animation
   public void play() {
      startTime = new Date().getTime()+500;
      alpha.setStartTime(startTime);
      rotPosInt.setEnable(true);
   }
   
   // Pause animation
   public void pause() {
      timeStamp = new Date().getTime() - startTime;
      rotPosInt.setEnable(false);
   }
   
   // Resume animation
   public void resume() {
      startTime = new Date().getTime() - timeStamp;
      alpha.setStartTime(startTime);
      rotPosInt.setEnable(true);
   }
   
   // Stop animation
   public void stop() {
      rotPosInt.setEnable(false);
   }  
   
   // Returns the id
   public String getId() {
      return id;
   }
   
   // Sets the id to s
   public void setId(String s) {
      id = s;
   }
   
   // Returns the length of the vehicle
   public float getLength() {
      return 3.55f;
   }
   
   // Returns the width of the vehicle
   public float getWidth() {
      return 1.25f;
   }
   
   // Returns the height of the vehicle
   public float getHeight() {
      return 1.25f;
   }
   
   
}