// This class creates a level crossing.

// 	Name: LevelCrossing (class)
//	Author: Arjan Egges
//	Version: 1.0

package  se.lth.cs.carsimgui;


import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;

class LevelCrossing implements AccidentObject {

   // Properties
   public float xPos;
   public float yPos;
   public String id;
   
   // Private members
   private TransformGroup lcGeo;
   
   // Constructor
   public LevelCrossing() {
      xPos = 0.0f;
      yPos = 0.0f;
      id = "level crossing";
   }
   
   // Creates the geometry of the LevelCrossing
   public void createGeometry() {
      Transform3D translate = new Transform3D();
      translate.set(new Vector3f(xPos,0.0f,yPos));
      lcGeo = new TransformGroup(translate);
  
      Color3f eColor    = new Color3f(0.8f, 0.8f, 0.8f);
      Color3f sColor    = new Color3f(1.0f, 1.0f, 1.0f);
      Color3f oColor    = new Color3f(1.0f, 1.0f, 1.0f);
      Material lcMat = new Material(oColor, eColor, oColor, sColor, 100.0f);
      lcMat.setLightingEnable(true);
      Appearance lcAppear = new Appearance();
      lcAppear.setMaterial(lcMat);
      
      // Add the levelcrossing, that consists of six boxes
      for (float incr=0.0f; incr<6; incr++) {
         translate.set(new Vector3f(0.0f,0.0f,-2.5f+incr));
         TransformGroup lcPart = new TransformGroup(translate);
         lcGeo.addChild(lcPart);
         lcPart.addChild(new Box(2.0f,0.01f,0.25f,Box.GENERATE_NORMALS,lcAppear));
      }  
   }
   
   // Returns the geometry of the LevelCrossing
   public TransformGroup getGeometry() {
      return lcGeo;
   }
   
   // Returns the id of the levelcrossing
   public String getId() {
      return id;
   }
   
   // Returns the collision radius
   // === NOTE: this is a ridiculous value, because a collision with a 
   //           levelcrossing is impossible! ===
   public float getCollisionRadius() {
      return -1.0f;
   }

   // Returns the x position of the levelcrossing
   public float getXPos() {
      return xPos;
   }
   
   // Returns the y position of the levelcrossing
   public float getYPos() {
      return yPos;
   }

}

