// This class adds the EventListener to the main window and calls the corresponding functions.

package  se.lth.cs.carsimgui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.io.*;

public class MainWindowEventListener implements ActionListener {
  
	MainWindow mw;
	MainWindowManager mwm;
	
	public MainWindowEventListener(MainWindow mainWindow, MainWindowManager mainWindowManager)
	{
		mw = mainWindow;
		mwm = mainWindowManager;
		
		
		//actionListener
		
		mw.menuProgramLanguageEnglish.addActionListener(this);
		mw.menuProgramLanguageDeutsch.addActionListener(this);
		mw.menuProgramLanguageEspanol.addActionListener(this);
		mw.menuProgramLanguageFrancais.addActionListener(this);
		mw.menuProgramLanguageSvenska.addActionListener(this);
		mw.menuProgramExit.addActionListener(this);
		mw.menuReportNew.addActionListener(this);
		mw.menuReportSave.addActionListener(this);
		mw.menuReportDelete.addActionListener(this);
		mw.menuReportGenerateXML.addActionListener(this);
		mw.menuXMLdocNew.addActionListener(this);
		mw.menuXMLdocSave.addActionListener(this);
		mw.menuXMLdocDelete.addActionListener(this);
		mw.menuXMLdocValidate.addActionListener(this);
		mw.menu3DStart.addActionListener(this);
		mw.menu3DPreferences.addActionListener(this);
		mw.menu3DDemoA.addActionListener(this);
		mw.menu3DDemoB.addActionListener(this);
		mw.menuHelpHelp.addActionListener(this);
		mw.menuHelpAbout.addActionListener(this);
		
		mw.newReportButton.addActionListener(this);
		mw.saveReportButton.addActionListener(this);
		mw.generateXMLButton.addActionListener(this);
		mw.newXMLdocButton.addActionListener(this);
		mw.saveXMLdocButton.addActionListener(this);
		mw.parseXMLdocButton.addActionListener(this);
		mw.demoAButton.addActionListener(this);
		mw.demoBButton.addActionListener(this);
		mw.backgroundButton.addActionListener(this);
		mw.visualizeButton.addActionListener(this);
		
		
		// MouseListener for list1 (reports) and list2 (XMLdocs)
		// If the user clicks on an item in a list, the mouseClicked function is called.
		// Example for list1 (reports):
		//
		// If one or several entries were deselected, the actual content of the text area is saved in textAreaReports[]
		// now every list entry is checked. If it is selected AFTER the click, wasReportSelected[i] is set to TRUE
		// if it is not selected but it was before, it is checked if there were made modifications to then text in the text area.
		// if "yes", the reports were modified (reportsModified = TRUE)
		// So if the reports were modified, a window appears that askes if the changes should be dropped ore saved.
		// At the end iterate list1, check if entry is selected and write the file content in the text area if required.
		
		MouseListener mouseListener = new MouseAdapter() {
			public void mouseClicked(MouseEvent event) {
				if (event.getSource() == mw.list1)
				{
					mwm.textArea1toStringArray();
					
					mwm.checkIfReportsModifiedThenSaveIfWanted();
					
					mw.textArea1.setText(mwm.getNewTextArea1String());
					
					mwm.writeListToBooleanArray(mw.list1,mwm.wasReportSelected);
				}
				else if (event.getSource() == mw.list2)
				{
					mwm.textArea2toStringArray();
					
					mwm.checkIfXMLdocsModifiedThenSaveIfWanted();
					
					mw.textArea2.setText(mwm.getNewTextArea2String());
					
					mwm.writeListToBooleanArray(mw.list2,mwm.wasXMLdocSelected);
				}
			}
		};
		mw.list1.addMouseListener(mouseListener);
		mw.list2.addMouseListener(mouseListener);
		
	}

	//############################################################### Event handling
	
	public void actionPerformed (ActionEvent event) {
		
		if (event.getSource() == mw.menuProgramLanguageEnglish)
		{
			Text.setTextes(Messenger.properties.ENGLISH,mw);
		}
		else if (event.getSource() == mw.menuProgramLanguageDeutsch)
		{
			Text.setTextes(Messenger.properties.DEUTSCH,mw);
		}
		else if (event.getSource() == mw.menuProgramLanguageEspanol)
		{
			Text.setTextes(Messenger.properties.ESPANOL,mw);
		}
		else if (event.getSource() == mw.menuProgramLanguageFrancais)
		{
			Text.setTextes(Messenger.properties.FRANCAIS,mw);
		}
		else if (event.getSource() == mw.menuProgramLanguageSvenska)
		{
			Text.setTextes(Messenger.properties.SVENSKA,mw);
		}
		else if (event.getSource() == mw.menuProgramExit)
		{
			// implementieren: Language, Background, Textures abspeichern.
			Messenger.saveProperties();
			System.exit(0);
		}
		else if ((event.getSource() == mw.menuReportNew) || (event.getSource() == mw.newReportButton))
		{
			mw.list1.clearSelection();
			mwm.textArea1toStringArray();
			mwm.checkIfReportsModifiedThenSaveIfWanted();
			mw.textArea1.setText("");
			mwm.writeListToBooleanArray(mw.list1,mwm.wasReportSelected);
		}
		else if ((event.getSource() == mw.menuReportSave) || (event.getSource() == mw.saveReportButton))
		{
			if (mw.list1.isSelectionEmpty())
			{
				mwm.saveNewReport();
			}
			else
			{
				mwm.textArea1toStringArray();
				mwm.modifiedReports2Files();
			}
		}
		else if (event.getSource() == mw.menuReportDelete)
		{
			mwm.deleteSelectedReports();
		}
		else if ((event.getSource() == mw.menuReportGenerateXML) || (event.getSource() == mw.generateXMLButton))
		{
			mwm.generateXMLfromTextArea1();
		}
		else if ((event.getSource() == mw.menuXMLdocNew) || (event.getSource() == mw.newXMLdocButton))
		{
			mw.list2.clearSelection();
			mwm.textArea2toStringArray();
			mwm.checkIfXMLdocsModifiedThenSaveIfWanted();
			if (mw.textArea2.getText().equals(""))
			{
				mw.textArea2.setText(mwm.newXMLdoc);
			}
			else
			{
				mw.textArea2.setText("");
			}
			mwm.writeListToBooleanArray(mw.list2, mwm.wasXMLdocSelected);
		}
		else if ((event.getSource() == mw.menuXMLdocSave) || (event.getSource() == mw.saveXMLdocButton))
		{
			if (mw.list2.isSelectionEmpty())
			{
				mwm.saveNewXMLdoc();
			}
			else
			{
				mwm.textArea2toStringArray();
				mwm.modifiedXMLdocs2Files();
			}
		}
		else if (event.getSource() == mw.menuXMLdocDelete)
		{
			mwm.deleteSelectedXMLdocs();
		}
		else if ((event.getSource() == mw.menuXMLdocValidate) || (event.getSource() == mw.parseXMLdocButton))
		{
			mwm.checkTextArea2();
		}
		else if ((event.getSource() == mw.menu3DStart) || (event.getSource() == mw.visualizeButton))
		{
			if (mwm.checkIfMaxOneXMLdocSelected())
			{
				mwm.visualizeTextArea2();
			}
			else
			{
				JOptionPane.showMessageDialog(mw, Text.at(47),"",JOptionPane.ERROR_MESSAGE);
			}
		}
		else if ((event.getSource() == mw.menu3DDemoA) || (event.getSource() == mw.demoAButton))
		{
			new DemoOne();
		}
		else if ((event.getSource() == mw.menu3DDemoB) || (event.getSource() == mw.demoBButton))
		{
			new DemoTwo();
		}
		else if (event.getSource() == mw.menu3DPreferences)
		{
			new PropertyFrame(mwm);
		}
		else if (event.getSource() == mw.menuHelpAbout)
		{
			MainWindow_About dlg = new MainWindow_About(mw);
			mwm.showWindowAtRightPlace(dlg);
			dlg.setModal(true);
		}
		else if (event.getSource() == mw.menuHelpHelp)
		{
			HelpFrame hf = new HelpFrame();
			mwm.showWindowAtRightPlace(hf);
		}



		else if (event.getSource() == mw.backgroundButton)
		{
			mwm.setBackgroundTextureProperties();
		}
	}
	
}