// This class provides all functions needed for executing the events in MainWindow
// 
// Function Overview:
// 
// public void checkIfReportsModifiedThenSaveIfWanted()
// public void checkIfXMLdocsModifiedThenSaveIfWanted()
// 
// public boolean checkIfMaxOneXMLdocSelected()
// 
// public void checkTextArea2()
// 
// public void deleteSelectedReports()
// public void deleteSelectedXMLdocs()
// 
// public void generateXMLfromTextArea1()
// private String generateXML(String reportString)
// 
// public String getNewTextArea1String()
// public String getNewTextArea2String()
// 
// public void modifiedReports2Files()
// public void modifiedXMLdocs2Files()
// 
// public void saveFileInFolderCarsimdata(String filename, String content) throws Exception
// public void saveNewReport()
// public void saveNewXMLdoc()
// 
// public void setBackgroundTextureProperties()
// 
// private void setbackModifiedReports()
// private void setbackModifiedXMLdocs()
// 
// public void setReportStrings()
// public void setXMLdocStrings()
// 
// public void showWindowAtRightPlace(Window d)
// 
// public void textArea1toStringArray()
// public void textArea2toStringArray()
// 
// public void visualizeTextArea2()
// 
// public void writeListToBooleanArray(JList list, boolean[] booleanArray)



package  se.lth.cs.carsimgui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.io.*;
import java.util.Iterator;
import java.util.Vector;
//import se.lth.cs.carsim.*;




public class MainWindowManager {
  
	MainWindow mw;
	MainWindowStatusManager mwsm;
	FileFilter ff;
	FormalismChecker fc;
	FormalismParser fp;
	
	private String parseText = "";
	private AccidentUniverse au;
	private boolean errors = false;
	
	public String[] fileReports;
	public String[] textAreaReports;
	public boolean[] wasReportSelected;
	
	public String[] fileXMLdocs;
	public String[] textAreaXMLdocs;
	public boolean[] wasXMLdocSelected;
	
	private int backgroundTextureStatus = 1;
	
	String newXMLdoc = "<?xml version=\"1.0\"?>\n<!DOCTYPE accident SYSTEM \"accident.dtd\">\n<accident>\n\t<staticObjects>\n\t\t<road kind=\"crossroad|straightroad|turn_left|turn_right\" />\n\t\t<tree id=\"tree1\">\n\t\t\t<coords x=\"0.0\" y=\"0.0\" />\n\t\t</tree>\n\t\t<sign kind=\"stop\">\n\t\t\t<coords x=\"0.0\" y=\"0.0\" />\n\t\t</sign>\n\t\t<trafficLight id=\"trafficLight1\" colorType=\"red|orange|green|inactive\">\n\t\t\t<coords x=\"0.0\" y=\"0.0\" />\n\t\t</trafficLight>\n\t\t<levelcrossing>\n\t\t\t<coords x=\"0.0\" y=\"0.0\" />\n\t\t</levelcrossing>\n\t</staticObjects>\n\t<dynamicObjects>\n\t\t<vehicle id=\"vehicle1\" kind=\"car|truck\" initDirection=\"north|east|south|west\" >\n\t\t\t<startSign>\n\t\t\t\tname\n\t\t\t</startSign>\n\t\t\t<endSign>\n\t\t\t\tname\n\t\t\t</endSign>\n\t\t\t<eventChain>\n\t\t\t\t<event kind=\"driving_forward|turn_left|turn_right|stop|overtake|change_lane_left|change_lane_right\" critical=\"yes|no\" />\n\t\t\t</eventChain>\n\t\t</vehicle>\n\t</dynamicObjects>\n\t<collisions>\n\t\t<collision>\n\t\t\t<actor id=\"\" side=\"front|rear|leftside|rightside|unknown\" />\n\t\t\t<victim id=\"\" side=\"front|rear|leftside|rightside|unknown\" />\n\t\t\t<coords x=\"0.0\" y=\"0.0\" />\n\t\t</collision>\n\t</collisions>\n</accident>";
	

	public MainWindowManager(MainWindow mainWindow, MainWindowStatusManager mainWindowStatusManager,FileFilter fileFilter)
	{
		mw = mainWindow;
		mwsm = mainWindowStatusManager;
		ff = fileFilter;
		fc = new FormalismChecker();

		if (Messenger.properties.createEnvironment)
		{
			if (Messenger.properties.textureOn)
			{
				mw.backgroundButton.setIcon(mw.backgroundIcon4);
				backgroundTextureStatus = 4;
			}
			else
			{
				mw.backgroundButton.setIcon(mw.backgroundIcon3);
				backgroundTextureStatus = 3;
			}
		}
		else
		{
			if (Messenger.properties.textureOn)
			{
				mw.backgroundButton.setIcon(mw.backgroundIcon2);
				backgroundTextureStatus = 2;
			}
			else
			{
				mw.backgroundButton.setIcon(mw.backgroundIcon1);
				backgroundTextureStatus = 1;
			}
		}
	}

	
	
	
	// The name of the function should say enough :-)
	// But although a detailed decription:
	// 
	// checks first, if there are reports, that WERE selected, but AREN'T anymore AND that were modified
	// The modification is checked by comparing the entries of textAreaReports[i] and fileReports[i],
	// so the textArea1 has to written in Strings first using the function textArea1toStringArray().
	//
	// If the were no reports selected, it is checked if the textArea1 is modified (s.th. different from "")
	// 
	// If there are modifications, a JOptionPane appears if the changes should be saved or not.
	
	
	public void checkIfReportsModifiedThenSaveIfWanted()
	{
		boolean reportsModified = false;
		boolean wasNoReportSelected = true; // => new report (maybe) in textArea1

		for (int i=0; i < mw.list1.getModel().getSize(); i++)
		{
			if (wasReportSelected[i]) // if report was selected...
			{
				wasNoReportSelected = false;
				if (!mw.list1.isSelectedIndex(i)) // ...but is not anymore
				{
					if (!textAreaReports[i].equals(fileReports[i]))
					{
						reportsModified = true;
					}
				}
			}
		}
		
		// check if there is a new report written in the textarea, which has to be saved first
		if (wasNoReportSelected)
		{
			if (!mw.textArea1.getText().equals(""))
			{
				reportsModified = true; // => new (not empty) report in textArea1
			}
		}
			
		if (reportsModified)
		{
			Object[] options = {Text.at(33),Text.at(34)}; // "yes", "no"
			int answer = JOptionPane.showOptionDialog(mw,
								Text.at(35),
								Text.at(36),
								JOptionPane.YES_NO_OPTION,
								JOptionPane.QUESTION_MESSAGE,
								null,     //don't use a custom Icon
								options,  //the titles of buttons
								options[0]);
			if (wasNoReportSelected)
			{
				if (answer == JOptionPane.YES_OPTION)
				{
				saveNewReport();
				}
			}
			else
			{
				if (answer == JOptionPane.YES_OPTION)
				{
					modifiedReports2Files();
				}
				else
				{
					setbackModifiedReports();
				}
			}
		}
	}
	
	
	public void checkIfXMLdocsModifiedThenSaveIfWanted()
	{
		boolean xmldocsModified = false;
		boolean wasNoXMLdocSelected = true; // => new XML doc (maybe) in textArea2

		for (int i=0; i < mw.list2.getModel().getSize(); i++)
		{
			if (wasXMLdocSelected[i]) // if XML doc was selected...
			{
				wasNoXMLdocSelected = false;
				if (!mw.list2.isSelectedIndex(i)) // ...but is not anymore
				{
					if (!textAreaXMLdocs[i].equals(fileXMLdocs[i]))
					{
						xmldocsModified = true;
					}
				}
			}
		}
		
		// check if there is a new report written in the textarea, which has to be saved first
		if (wasNoXMLdocSelected)
		{
			// if textArea2 is not empty and doesn't have the standard XML doc template as its content...
			if (!mw.textArea2.getText().equals(newXMLdoc) && !mw.textArea2.getText().equals(""))
			{
				// ... than it's modified.
				xmldocsModified = true;
			}
		}
			
		if (xmldocsModified)
		{
			Object[] options = {Text.at(33),Text.at(34)}; // "yes", "no"
			int answer = JOptionPane.showOptionDialog(mw,
								Text.at(41),
								Text.at(36),
								JOptionPane.YES_NO_OPTION,
								JOptionPane.QUESTION_MESSAGE,
								null,     //don't use a custom Icon
								options,  //the titles of buttons
								options[0]);
			if (wasNoXMLdocSelected)
			{
				if (answer == JOptionPane.YES_OPTION)
				{
				saveNewXMLdoc();
				}
			}
			else
			{
				if (answer == JOptionPane.YES_OPTION)
				{
					modifiedXMLdocs2Files();
				}
				else
				{
					setbackModifiedXMLdocs();
				}
			}
		}
	}
	
	
	
	public boolean checkIfMaxOneXMLdocSelected()
	{
		int counter = 0;
		for (int i=0; i < mw.list2.getModel().getSize(); i++)
		{
			if (mw.list2.isSelectedIndex(i)) 
			{
				counter++;
			}
		}
		if (counter>1) return false;
		return true;
	}
	
	
	
	public void checkTextArea2()
	{
		textArea2toStringArray();
		
		boolean newXMLdoc = true;
		boolean xmldocValid = true;
		
		for(int i = 0; i < mw.list2.getModel().getSize(); i++)
		{
			if(wasXMLdocSelected[i])
			{
				newXMLdoc = false;
				if (!fc.parse(textAreaXMLdocs[i]))
				{
					xmldocValid = false;
				}
			}
		}
		if (newXMLdoc)
		{
			if (!fc.parse(mw.textArea2.getText()))
			{
				xmldocValid = false;
			}
		}
		
		if(xmldocValid)
		{
			mwsm.setStatus(mwsm.OK,Text.at(53));
		}
		else
		{
			mwsm.setStatus(mwsm.ERROR,Text.at(54));
		}
	}
	
	
	
	public void deleteSelectedReports()
	{
		if (mw.list1.isSelectionEmpty())
		{
			mwsm.setStatus(mwsm.ERROR, Text.at(56));
		}
		else
		{
			Object[] options = {Text.at(33),Text.at(34)}; // "yes", "no"
			int answer = JOptionPane.showOptionDialog(mw,
								Text.at(57), // Delete all selected reports?
								Text.at(58), // Delete reports
								JOptionPane.YES_NO_OPTION,
								JOptionPane.QUESTION_MESSAGE,
								null,     //don't use a custom Icon
								options,  //the titles of buttons
								options[0]);
			if (answer == JOptionPane.YES_OPTION)
			{
				boolean deletionSuccessful = true;
				for (int i=0; i < mw.list1.getModel().getSize(); i++)
				{
					if (mw.list1.isSelectedIndex(i))
					{
						File file = new File("carsimdata/"+mw.list1.getModel().getElementAt(i)+".ana");
						if (!file.delete())
						{
							deletionSuccessful = false;
						}
						wasReportSelected[i] = false;
					}
				}
				if(deletionSuccessful)
				{
					mwsm.setStatus(mwsm.OK, Text.at(59));
				}
				else
				{
					mwsm.setStatus(mwsm.ERROR, Text.at(60));
				}
				mw.textArea1.setText("");
				mw.list1.setListData(ff.getFileArray(".ana"));
				setReportStrings();
			}
		}
	}
	
	
	
	public void deleteSelectedXMLdocs()
	{
		if (mw.list2.isSelectionEmpty())
		{
			mwsm.setStatus(mwsm.ERROR, Text.at(61));
		}
		else
		{
			Object[] options = {Text.at(33),Text.at(34)}; // "yes", "no"
			int answer = JOptionPane.showOptionDialog(mw,
								Text.at(62), // Delete all selected reports?
								Text.at(63), // Delete reports
								JOptionPane.YES_NO_OPTION,
								JOptionPane.QUESTION_MESSAGE,
								null,     //don't use a custom Icon
								options,  //the titles of buttons
								options[0]);
			if (answer == JOptionPane.YES_OPTION)
			{
				boolean deletionSuccessful = true;
				for (int i=0; i < mw.list2.getModel().getSize(); i++)
				{
					if (mw.list2.isSelectedIndex(i))
					{
						File file = new File("carsimdata/"+mw.list2.getModel().getElementAt(i)+".ade");
						if (!file.delete())
						{
							deletionSuccessful = false;
						}
						wasXMLdocSelected[i] = false;
					}
				}
				if(deletionSuccessful)
				{
					mwsm.setStatus(mwsm.OK, Text.at(64));
				}
				else
				{
					mwsm.setStatus(mwsm.ERROR, Text.at(65));
				}
				mw.textArea2.setText("");
				mw.list2.setListData(ff.getFileArray(".ade"));
				setXMLdocStrings();
			}
		}
	}
	
	
	
	public void generateXMLfromTextArea1()
	{
		// if there is text in the textArea1...
		
		if (!mw.textArea1.getText().equals(""))
		{
			mw.list2.clearSelection();
			textArea2toStringArray();
			// check, if there are modified XML docs in TextArea2, which have to be saved first.
			checkIfXMLdocsModifiedThenSaveIfWanted();
			mw.textArea2.setText("");
			writeListToBooleanArray(mw.list2,wasXMLdocSelected); // now every entry is "false"
			
			if (mw.list1.isSelectionEmpty())
			{
				mw.textArea2.setText( generateXML( mw.textArea1.getText() ) );
			}
			else
			{
				// gererate XML from the selected reports
				// write the new XML docs in files
				// use the same name as the report with the ending ".ade" instead of ".ana"
				
				Vector vSelectedXMLdocs = new Vector(5);
				
				for (int i=0; i < mw.list1.getModel().getSize(); i++)
				{
					if (wasReportSelected[i]) // if report in list1 is select
					{
						try
						{
							// save generated XML doc in file <reportname>.ade
							String parseResult = generateXML(textAreaReports[i]);
							//String parseResult = "<xml doc>\n\t<accident/>\n</xmldoc>";
							
							saveFileInFolderCarsimdata(mw.list1.getModel().getElementAt(i)+".ade",parseResult);
							
							// set new: list2, fileXMLdocs[], textAreaXMLdocs[] 
							mw.list2.setListData(ff.getFileArray(".ade"));
							setXMLdocStrings();
							
							// write new list2 entry to vector and select the new entries
							vSelectedXMLdocs.add(mw.list1.getModel().getElementAt(i));
							Iterator iterator = vSelectedXMLdocs.iterator();
							while(iterator.hasNext())
							{
								mw.list2.setSelectedValue((String)iterator.next(), false);
							}
							
							// register list2-selection in wasXMLdocSelected[]
							writeListToBooleanArray(mw.list2,wasXMLdocSelected);
							mw.textArea2.setText(getNewTextArea2String());
						}
						catch (Exception e)
						{
							System.out.println("Error in MainWindowManager.generateXMLfromTextArea1(): "+e);
							mwsm.setStatus(mwsm.ERROR, Text.at(52));
						}
					}
				}				
			}
		}
		
	}
	
	
	
	// private void generateXML(String reportString)
	private String generateXML(String reportString)
	{
		/*
		if( WNDictionary.isNotInitialized() )
		{
			mwsm.setStatus(mwsm.INFO, Text.at(70), Integer.MAX_VALUE);
			WNDictionary.init();
		
		}
		mwsm.setStatus(mwsm.INFO, Text.at(68), Integer.MAX_VALUE);
		      
		Dispatch dispatch = new Dispatch();
		String XMLString = dispatch.extractInformationToXML(reportString);
		mwsm.setStatus(mwsm.INFO, Text.at(69));
		return XMLString;
		*/
		return "This option does not work.";
		// Sorry, Ola, but I have to comment it out..!
		/*
		final String report = reportString; // needs to be final, because the inner class SwingWorker accesses to it.
		final SwingWorker worker = new SwingWorker()
		{
			String XMLString = null;
			Dispatch dispatch = null;
		    
			public Object construct()
			{
				if( WNDictionary.isNotInitialized() )
				{
					mwsm.setStatus(mwsm.INFO, Text.at(70), Integer.MAX_VALUE);
					WNDictionary.init();
				}
				mwsm.setStatus(mwsm.INFO, Text.at(68), Integer.MAX_VALUE);
		       
				dispatch = new Dispatch();
				XMLString = new String( dispatch.extractInformationToXML(report));
				return XMLString; //return value not used by this program
			}
		    
			//Runs on the event-dispatching thread.
			public void finished()
			{
				mw.textArea2.setText(XMLString);
				mwsm.setStatus(mwsm.INFO, Text.at(69), 30);
			}
		};
		
		worker.start();  //required for SwingWorker 3
		System.out.println("Thread started extracting information to construct a XML template from a text");
		*/
	}	   


	
	
	public String getNewTextArea1String()
	{
		String intoTextArea = "";
		for (int i=0; i < mw.list1.getModel().getSize(); i++)
		{
			if (mw.list1.isSelectedIndex(i)) // if report in list1 is select
			{
				intoTextArea += "<"+mw.list1.getModel().getElementAt(i)+">\n";
				intoTextArea += textAreaReports[i]+"\n";
				intoTextArea += "</"+mw.list1.getModel().getElementAt(i)+">\n";
				
			}
		}
		return intoTextArea;
	}
	
	
	
	public String getNewTextArea2String()
	{
		String intoTextArea = "";
		for (int i=0; i < mw.list2.getModel().getSize(); i++)
		{
			if (mw.list2.isSelectedIndex(i)) // if report in list1 is select
			{
				intoTextArea += "<START "+mw.list2.getModel().getElementAt(i)+">\n";
				intoTextArea += textAreaXMLdocs[i]+"\n";
				intoTextArea += "<END "+mw.list2.getModel().getElementAt(i)+">\n";
				
			}
		}
		return intoTextArea;
	}
	
	
	
	// writes all reports from the String Array textAreaReports[] to fileReports[] and to the files as well
	
	public void modifiedReports2Files()
	{
		try
		{
			for(int i = 0; i < mw.list1.getModel().getSize(); i++) {
				if (wasReportSelected[i])
				{
					fileReports[i] = textAreaReports[i];
					saveFileInFolderCarsimdata(mw.list1.getModel().getElementAt(i)+".ana", fileReports[i]);
				}
			}
			mwsm.setStatus(mwsm.OK,Text.at(50));
		}
		catch (Exception e)
		{
			mwsm.setStatus(mwsm.ERROR,Text.at(52));
		}
	}
	
	
	
	public void modifiedXMLdocs2Files()
	{
		try
		{
			for(int i = 0; i < mw.list2.getModel().getSize(); i++) {
				if (wasXMLdocSelected[i])
				{
					fileXMLdocs[i] = textAreaXMLdocs[i];
					saveFileInFolderCarsimdata(mw.list2.getModel().getElementAt(i)+".ade", fileXMLdocs[i]);
				}
			}
			mwsm.setStatus(mwsm.OK,Text.at(51));
		}
		catch (Exception e)
		{
			mwsm.setStatus(mwsm.ERROR,Text.at(52));
		}
	}
	
	
	
	public void saveFileInFolderCarsimdata(String filename, String content) throws Exception
	{
		FileOutputStream fos;
		Writer out;
		fos = new FileOutputStream("carsimdata/"+filename);
		out = new OutputStreamWriter(fos, "ISO-8859-1");
		out.write(content);
		out.close();
	}
	
	
	
	public void saveNewReport()
	{
		try
		{
			String filename = (String) JOptionPane.showInputDialog(mw,
						Text.at(37),
						Text.at(38),
						JOptionPane.YES_NO_OPTION
						);
			if (filename != null)
			{
				filename.replaceAll(".ana","");
				saveFileInFolderCarsimdata(filename+".ana", mw.textArea1.getText());
				
				mw.list1.setListData(ff.getFileArray(".ana"));
				setReportStrings();
				mw.list1.setSelectedValue(filename, true);
				wasReportSelected[mw.list1.getSelectedIndex()] = true;
				mw.textArea1.setText("<"+filename+">\n"+mw.textArea1.getText()+"\n</"+filename+">");
				mwsm.setStatus(mwsm.OK,Text.at(50));
			}
		}
		catch (Exception e)
		{
			System.out.println("Error in MainWindowManager.saveNewReport: "+e);
		}
	}
	
	
	
	public void saveNewXMLdoc()
	{
		try
		{
			String filename = (String) JOptionPane.showInputDialog(mw,
						Text.at(42),
						Text.at(38),
						JOptionPane.YES_NO_OPTION
						);
			if (filename != null)
			{
				filename.replaceAll(".ade","");
				saveFileInFolderCarsimdata(filename+".ade", mw.textArea2.getText());
				
				mw.list2.setListData(ff.getFileArray(".ade"));
				setXMLdocStrings();
				mw.list2.setSelectedValue(filename, true);
				wasXMLdocSelected[mw.list2.getSelectedIndex()] = true;
				mw.textArea2.setText("<START "+filename+">\n"+mw.textArea2.getText()+"\n<END "+filename+">");
				mwsm.setStatus(mwsm.OK,Text.at(51));
			}
		}
		catch (Exception e)
		{
			System.out.println("Error in MainWindowManager.saveNewXMLdoc: "+e);
		}
	}
	
	
	
	public void setBackgroundTextureProperties()
	{
		backgroundTextureStatus++;
		if (backgroundTextureStatus > 4)
			backgroundTextureStatus = 1;
		if (backgroundTextureStatus == 1)
		{
			Messenger.properties.createEnvironment = false;
			Messenger.properties.textureOn = false;
			mw.backgroundButton.setIcon(mw.backgroundIcon1);
		}
		if (backgroundTextureStatus == 2)
		{
			Messenger.properties.createEnvironment = false;
			Messenger.properties.textureOn = true;
			mw.backgroundButton.setIcon(mw.backgroundIcon2);
		}
		if (backgroundTextureStatus == 3)
		{
			Messenger.properties.createEnvironment = true;
			Messenger.properties.textureOn = false;
			mw.backgroundButton.setIcon(mw.backgroundIcon3);
		}
		if (backgroundTextureStatus == 4)
		{
			Messenger.properties.createEnvironment = true;
			Messenger.properties.textureOn = true;
			mw.backgroundButton.setIcon(mw.backgroundIcon4);
		}
	}
	
	
	
	private void setbackModifiedReports()
	{
		for(int i = 0; i < mw.list1.getModel().getSize(); i++)
		{
			textAreaReports[i] = fileReports[i];
		}
	}
	
	
	
	private void setbackModifiedXMLdocs()
	{
		for(int i = 0; i < mw.list2.getModel().getSize(); i++)
		{
			textAreaXMLdocs[i] = fileXMLdocs[i];
		}
	}
	
	
	
	// takes the entries of list1 (filenames of the reports)
	// loads the contents of the files and saves them into fileReports[]
	// initializes reportSelected[i] = false
	
	public void setReportStrings()
	{
		//allocates a little bit more space for creating new reports
		int listSizePlus10 	= mw.list1.getModel().getSize() + 10; 
		wasReportSelected 	= new boolean[listSizePlus10];
		fileReports 		= new String[listSizePlus10];
		textAreaReports 	= new String[listSizePlus10];
		
		
		try
		{
			File inputFile;
			FileReader in;
			
			for(int i = 0; i < mw.list1.getModel().getSize(); i++) {
				wasReportSelected[i] = false;
				inputFile = new File("carsimdata/"+mw.list1.getModel().getElementAt(i)+".ana");
				in = new FileReader(inputFile);
				int c;
				String temp = "";
				while ((c = in.read()) != -1)
				{
					temp = temp + (char)c;
				}
				fileReports[i] = temp;
				textAreaReports[i] = temp;
				in.close();
			}
		}
		catch (Exception e) { System.out.println("Error in MainWindowManager.setReportStrings(): "+e); }
	}
	
	
	
	public void setXMLdocStrings()
	{
		//allocates a little bit more space for creating new XML docs
		int listSizePlus10 	= mw.list1.getModel().getSize() + 10;
		wasXMLdocSelected 	= new boolean[listSizePlus10];
		fileXMLdocs 		= new String[listSizePlus10];
		textAreaXMLdocs 	= new String[listSizePlus10];
		
		
		try
		{
			File inputFile;
			FileReader in;
			
			for(int i = 0; i < mw.list2.getModel().getSize(); i++) {
				wasXMLdocSelected[i] = false;
				inputFile = new File("carsimdata/"+mw.list2.getModel().getElementAt(i)+".ade");
				in = new FileReader(inputFile);
				int c;
				String temp = "";
				while ((c = in.read()) != -1)
				{
					temp = temp + (char)c;
				}
				fileXMLdocs[i] = temp;
				textAreaXMLdocs[i] = temp;
				in.close();
			}
		}
		catch (Exception e) { System.out.println("Error in MainWindowManager.setXMLdocStrings(): "+e); }
	}
	
	
	
	public void showWindowAtRightPlace(Window d)
	{
		Dimension dSize = d.getPreferredSize();
		Dimension frmSize = mw.getSize();
		Point loc = mw.getLocation();
		d.setLocation((frmSize.width - dSize.width) / 2 + loc.x, (frmSize.height - dSize.height) / 2 + loc.y);
		d.show();
	}
	
	
	
	// if an item of the list1 (reports) was deselected, the content of the textarea has to be verified, if it
	// was changed by the user. This function parses the textareas content and saves the textes into textAreaReports
	
	public void textArea1toStringArray()
	{
		String textAreaContent = "";
		String temp = "";
		try { textAreaContent = mw.textArea1.getText(); }
		catch(Exception e) { System.out.println("Error in MainWindowManager.textArea1toModifiedXMLdocs: "+e); }
		
		for(int i = 0; i < mw.list1.getModel().getSize(); i++)
		{
			if(wasReportSelected[i])
			{
				temp = (String)mw.list1.getModel().getElementAt(i);
				try
				{
					textAreaReports[i] = textAreaContent.substring(
						textAreaContent.indexOf("<"+temp+">")+3+temp.length(),
						textAreaContent.indexOf("</"+temp+">")-1
						);
				} 
				catch (IndexOutOfBoundsException e)
				{
					mwsm.setStatus(mwsm.ERROR,Text.at(48)+" <"+temp+">...</"+temp+"> "+Text.at(49));
					textAreaReports[i] = fileReports[i];
				}
			}
		}
	}
	
	
	
	public void textArea2toStringArray()
	{
		String textAreaContent = "";
		String temp = "";
		try { textAreaContent = mw.textArea2.getText(); }
		catch(Exception e) { System.out.println("Error in MainWindowManager.textArea2toStringArray: "+e); }
		
		for(int i = 0; i < mw.list2.getModel().getSize(); i++)
		{
			if(wasXMLdocSelected[i])
			{
				temp = (String)mw.list2.getModel().getElementAt(i);
				try
				{
					//System.out.println("textAreaContent: "+textAreaContent+" temp: "+temp+" "+(textAreaContent.indexOf("<"+temp+">")+2+temp.length())+" "+textAreaContent.indexOf("</"+temp+">"));
					//System.out.println(textAreaContent.indexOf("<"+temp+">")+2+temp.length());
					textAreaXMLdocs[i] = textAreaContent.substring(
						textAreaContent.indexOf("<START "+temp+">")+9+temp.length(),
						textAreaContent.indexOf("<END "+temp+">")-1
						);
				} 
				catch (IndexOutOfBoundsException e)
				{
					mwsm.setStatus(mwsm.ERROR,Text.at(48)+" <START "+temp+">...<END "+temp+"> "+Text.at(49));
					textAreaXMLdocs[i] = fileXMLdocs[i];
				}
			}
		}
	}
	
	
	
	public void visualizeTextArea2()
	{
		mwsm.setStatus(mwsm.INFO,Text.at(55));
		
		// get the XMLdoc from the textArea
		// either with or without <START filename> tag
		
		String textAreaContent = "";
		String temp = "";
		try { textAreaContent = mw.textArea2.getText(); }
		catch(Exception e) { System.out.println("Error in MainWindowManager.textArea2toStringArray: "+e); }
		
		temp = (String)mw.list2.getSelectedValue();
		
		if (temp != null)
		{
			textAreaContent = textAreaContent.substring(
				textAreaContent.indexOf("<START "+temp+">")+9+temp.length(),
				textAreaContent.indexOf("<END "+temp+">")-1
				);
		}
		
		// if the XMLdoc is valid, generate the 3D scene
		
		if (fc.parse(textAreaContent))
		{
			fp = new FormalismParser();
			fp.parse(textAreaContent);
			errors = fp.errors();
			au = fp.getUniverse();
			
			// for debugging
			// au.getAccidentInfo().printAccidentInfo();
			
			if (errors)
			{
				Messenger.message("Could not create simulation due to parsing errors.\n");
				return;
			}
			// Plan the accident.
			AccidentPlanner planner = new AccidentPlanner(au.getAccidentInfo());
			planner.plan();
			if (planner.errors())
			{
				Messenger.message("Unable to plan accident. Aborting simulation.\n");
				return;
			}
			new Simulation(au);    
		}
		else
		{
			mwsm.setStatus(mwsm.ERROR,Text.at(54));	// The XML document is not valid.
		}
	}
	
	
	public void writeListToBooleanArray(JList list, boolean[] booleanArray)
	{
		for (int i=0; i < list.getModel().getSize(); i++)
		{
			if (list.isSelectedIndex(i)) // if entry in list is select
			{
				booleanArray[i] = true;
			}
			else
			{
				booleanArray[i] = false;
			}
		}
	}
}
