// This class is responsible for displaying error or other messages.
// A thread cleares the text in the statusbar after a while.
//
// To display a message i the status bar of the main window
// just call the funtion setStatus(...)
// e.g. mwsm.setStatus(mwsm.ERROR, "error while parsing report", 10);
// => Displays a red ERROR dot and the text above for 10 seconds in the status bar.
//	
// Since Version 1.1
// by Bastian Schulz

package  se.lth.cs.carsimgui;

import java.util.Timer;
import java.util.TimerTask;

public class MainWindowStatusManager
{
	Timer timer;
	MainWindow mw;
	private int seconds = 0;
	
	public final static int EMPTY   = 0; 
	public final static int INFO    = 1;
	public final static int OK      = 2;
	public final static int ERROR   = 3;
	
	
 	public MainWindowStatusManager(MainWindow mainWindow)
 	{
		mw = mainWindow;
		timer = new Timer();
		timer.schedule(new StatusTask(), 1000, 1000);
	}
	
	
	
	public void setStatus(int statuskind, String message, int duration)
	{
		mw.statusLabel.setText(message);
		if (statuskind == INFO)		{ mw.statusLED.setIcon(mw.statusInfoIcon); 	} // white dot
		else if (statuskind == OK) 	{ mw.statusLED.setIcon(mw.statusOkIcon); 	} // green dot
		else if (statuskind == ERROR) 	{ mw.statusLED.setIcon(mw.statusErrorIcon); 	} // red dot
		else 				{ mw.statusLED.setIcon(mw.statusEmptyIcon); 	} // transparent dot
		seconds = duration;
		
	}
	
	
	
	public void setStatus(int statuskind, String message)
	{
		setStatus(statuskind, message, 10);
	}
	
	
	
	public void stopTimer()
	{
		timer.cancel(); //Terminate the timer thread
	}
	
	
	
	class StatusTask extends TimerTask
	{
		public void run() {
			if (seconds > 0)
			{
				seconds--;
			}
			else if (seconds == 0)
			{
				mw.statusLabel.setText("");
				mw.statusLED.setIcon(mw.statusEmptyIcon);
			}
 		}
	}
}
