// This class creates a road of a certain kind.

// 	Name: Road (class)
//	Author: Arjan Egges
//	Version: 1.0


package  se.lth.cs.carsimgui;

import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;

class Road implements AccidentObject {
   
   // Property variables
   public boolean sideStripes;
   public boolean middleStripes;
   public boolean textureOn;
   public int type;
   public float length;
   public float width;
   
   // Private members
   private TransformGroup roadGeometry;
   private boolean subroadSpecified;
   private Road subRoad;
   private boolean northroadSpecified;
   private Road nRoad;
   private boolean southroadSpecified;
   private Road sRoad;
   private boolean eastroadSpecified;
   private Road eRoad;
   private boolean westroadSpecified;
   private Road wRoad;
   
   // Constructor
   public Road() {
      sideStripes = true;
      middleStripes = true;
      textureOn = true;
      type = Constant.STRAIGHTROAD;
      length = 200.0f;
      width = 6.0f;
      subroadSpecified = false;
      northroadSpecified = false;
      southroadSpecified = false;
      eastroadSpecified = false;
      westroadSpecified = false;
   }
   
   // Creates the geometry of the Road object
   public void createGeometry() {
      // Set the properties
      textureOn = Messenger.properties.textureOn;
      
      roadGeometry = new TransformGroup();
      
      Transform3D modelTransform = new Transform3D();
      Appearance stripeAppear = new Appearance();
      
      if (type == Constant.CROSSROAD) {
         // Create a crossroad object and add it to the scene graph
         // The crossroad is created from five roads
         Road northRoad;
         Road southRoad;
         Road eastRoad;
         Road westRoad;
         
         if (!northroadSpecified) {
            northRoad = new Road();
            // Set the properties
            northRoad.width = this.width;
            northRoad.length = (this.length - this.width)/2.0f;
            northRoad.sideStripes = this.sideStripes;
            northRoad.middleStripes = this.middleStripes;
            northRoad.textureOn = this.textureOn;
         } else
            northRoad = nRoad;
         
         if (!southroadSpecified) {
            southRoad = new Road();
            // Set the properties
            southRoad.width = this.width;
            southRoad.length = (this.length - this.width)/2.0f;
            southRoad.sideStripes = this.sideStripes;
            southRoad.middleStripes = this.middleStripes;
            southRoad.textureOn = this.textureOn;
         } else
            southRoad = sRoad;
         
         if (!eastroadSpecified) {
            eastRoad = new Road();
            // Set the properties
            eastRoad.width = this.width;
            eastRoad.length = (this.length - this.width)/2.0f;
            eastRoad.sideStripes = this.sideStripes;
            eastRoad.middleStripes = this.middleStripes;
            eastRoad.textureOn = this.textureOn;
         } else
            eastRoad = eRoad;
         
         if (!westroadSpecified) {
            westRoad = new Road();
            // Set the properties
            westRoad.width = this.width;
            westRoad.length = (this.length - this.width)/2.0f;
            westRoad.sideStripes = this.sideStripes;
            westRoad.middleStripes = this.middleStripes;
            westRoad.textureOn = this.textureOn;
         } else
            westRoad = wRoad;
         
         Road middleRoad = new Road();
         // Set the properties
         middleRoad.width = this.width;
         middleRoad.length = this.width; // The 'middle road' is actually the crossing itself.         
         middleRoad.sideStripes = false;
         middleRoad.middleStripes = false;
         middleRoad.textureOn = this.textureOn;
         
         // Create the geometries
         northRoad.createGeometry();
         southRoad.createGeometry();
         eastRoad.createGeometry();
         westRoad.createGeometry();
         middleRoad.createGeometry();
         
         // Transform all the roads (except the middleroad) properly.
         Transform3D rotate = new Transform3D();
         Transform3D translate = new Transform3D();
         
         rotate.rotY(Math.PI/2.0f);
         translate.set(new Vector3f(0.0f, 0.0f, -(northRoad.length+middleRoad.width)/2.0f));
         translate.mul(rotate);
         TransformGroup north = new TransformGroup(translate);
         north.addChild(northRoad.getGeometry());
         roadGeometry.addChild(north);
         
         rotate.rotY(-Math.PI/2.0f);
         translate.set(new Vector3f(0.0f, 0.0f, (southRoad.length+middleRoad.width)/2.0f));
         translate.mul(rotate);
         TransformGroup south = new TransformGroup(translate);
         south.addChild(southRoad.getGeometry());
         roadGeometry.addChild(south);
         
         translate.set(new Vector3f((eastRoad.length+middleRoad.width)/2.0f, 0.0f, 0.0f));
         TransformGroup east = new TransformGroup(translate);
         east.addChild(eastRoad.getGeometry());
         roadGeometry.addChild(east);
         
         rotate.rotY(Math.PI);
         translate.set(new Vector3f(-(westRoad.length+middleRoad.width)/2.0f, 0.0f, 0.0f));
         translate.mul(rotate);
         TransformGroup west = new TransformGroup(translate);
         west.addChild(westRoad.getGeometry());        
         roadGeometry.addChild(west);
         
         roadGeometry.addChild(middleRoad.getGeometry());
         
         
         
      } else if (type == Constant.STRAIGHTROAD) {         
         
         if (textureOn) {
            // Create the road, a textured XZ plane
            Shape3D roadPlane = null;
            roadPlane = new TexturedXYPlane("images/rock033_3.jpg",length,width);
            Transform3D rotate = new Transform3D();
            rotate.rotX(-Math.PI/2.0f);
            TransformGroup rt = new TransformGroup(rotate);
            rt.addChild(roadPlane);
            roadGeometry.addChild(rt);
          
         } else {
            // Create the road, a Box object
            Appearance rockAppear = new Appearance();
            Color3f eColor    = new Color3f(0.0f, 0.0f, 0.0f);
            Color3f sColor    = new Color3f(1.0f, 1.0f, 1.0f);
            Color3f oColor    = new Color3f(0.8f, 0.8f, 0.8f);
      
            Material rockMat = new Material(oColor, eColor, oColor, sColor, 100.0f);
            rockMat.setLightingEnable(true);
            rockAppear.setMaterial(rockMat);
            Transform3D translate = new Transform3D();
            translate.set(new Vector3f(0.0f,-0.1f,0.0f));
            TransformGroup tg = new TransformGroup(translate);
            tg.addChild(new Box(length/2.0f,0.1f,width/2.0f,Box.GENERATE_NORMALS,rockAppear));
            roadGeometry.addChild(tg);
         }
         
         if (middleStripes) {
            // Create the middle stripes. Little flat boxes.            
            Color3f streColor    = new Color3f(0.8f, 0.8f, 0.8f);
            Color3f strsColor    = new Color3f(1.0f, 1.0f, 1.0f);
            Color3f stroColor    = new Color3f(1.0f, 1.0f, 1.0f);
            Material stripeMat = new Material(stroColor, streColor, stroColor, strsColor, 100.0f);
            stripeMat.setLightingEnable(true);
            stripeAppear.setMaterial(stripeMat);
            float tr=-length/2.0f;     
            while (tr<length/2.0f) {
               modelTransform.set(new Vector3f(tr,0.0f,0.0f));
               TransformGroup transStr = new TransformGroup(modelTransform);
               tr += 1.5f;
               transStr.addChild(new Box(0.2f,0.01f,0.05f,Box.GENERATE_NORMALS,stripeAppear));
               roadGeometry.addChild(transStr);
         
            }
         }
         
         if (sideStripes) {
            // Create two large stripes along the side.            
            modelTransform.set(new Vector3f(0.0f,0.0f,(width/2.0f)-0.15f));
            TransformGroup strLong = new TransformGroup(modelTransform);
            strLong.addChild(new Box(length/2.0f,0.01f,0.05f,Box.GENERATE_NORMALS,stripeAppear));
            roadGeometry.addChild(strLong);
            modelTransform.set(new Vector3f(0.0f,0.0f,(-width/2.0f)+0.15f));
            TransformGroup strLong2 = new TransformGroup(modelTransform);
            strLong2.addChild(new Box(length/2.0f,0.01f,0.05f,Box.GENERATE_NORMALS,stripeAppear));
            roadGeometry.addChild(strLong2);
         }
      } else if (type == Constant.ROADSLEFT || type == Constant.ROADSRIGHT) {
      
         // This roadtype consists of two roads. One mainroad and one subroad. The subroad will have half the length of
         // the mainroad.
         Road mainroad = new Road();
         Road subroad;
         
         // Set the properties
         mainroad.textureOn = this.textureOn;
         mainroad.sideStripes = false;
         mainroad.middleStripes = this.middleStripes;
         mainroad.width = this.width;
         mainroad.length = this.length;
         
         if (!subroadSpecified) {  
            subroad = new Road();       
            subroad.textureOn = this.textureOn;
            subroad.sideStripes = this.sideStripes;
            subroad.middleStripes = this.middleStripes;
            subroad.width = this.width;
            subroad.length = this.length/2.0f;
         } else 
            subroad = subRoad;
            
         
         // Create the geometries
         mainroad.createGeometry();
         subroad.createGeometry();
         roadGeometry.addChild(mainroad.getGeometry());
         
         // Turn the subroad and translate it.
         Transform3D rotate = new Transform3D();
         Transform3D translate = new Transform3D();
         
         rotate.rotY(-Math.PI/2.0f);
         if (type == Constant.ROADSLEFT) 
            translate.set(new Vector3f(0.0f, 0.0f, -(subroad.length+mainroad.width)/2.0f));
         else
            translate.set(new Vector3f(0.0f, 0.0f, (subroad.length+mainroad.width)/2.0f));
         translate.mul(rotate);
         TransformGroup tf = new TransformGroup(translate);
         tf.addChild(subroad.getGeometry());
         roadGeometry.addChild(tf);
         
         
         if (sideStripes) {
         
            Color3f streColor    = new Color3f(0.8f, 0.8f, 0.8f);
            Color3f strsColor    = new Color3f(1.0f, 1.0f, 1.0f);
            Color3f stroColor    = new Color3f(1.0f, 1.0f, 1.0f);
            Material stripeMat = new Material(stroColor, streColor, stroColor, strsColor, 100.0f);
            stripeMat.setLightingEnable(true);
            stripeAppear.setMaterial(stripeMat);
            
            // Create two large stripes along the side of the mainroad, of which one is broken             
            if (type == Constant.ROADSLEFT)
               modelTransform.set(new Vector3f(0.0f,0.0f,(width/2.0f)-0.15f));
            else
               modelTransform.set(new Vector3f(0.0f,0.0f,(-width/2.0f)+0.15f));
            TransformGroup strLong = new TransformGroup(modelTransform);
            strLong.addChild(new Box(length/2.0f,0.01f,0.05f,Box.GENERATE_NORMALS,stripeAppear));
            roadGeometry.addChild(strLong);
            
            float lengthOfBS = (mainroad.length-subroad.width)/2.0f + 0.15f;
            if (type == Constant.ROADSLEFT)
               modelTransform.set(new Vector3f(-((lengthOfBS+subroad.width)/2.0f-0.15f), 0.0f,(-width/2.0f)+0.15f));
            else
               modelTransform.set(new Vector3f(-((lengthOfBS+subroad.width)/2.0f-0.15f), 0.0f,(width/2.0f)-0.15f));
            TransformGroup strLong2broken = new TransformGroup(modelTransform);
            strLong2broken.addChild(new Box(lengthOfBS/2.0f,0.01f,0.05f,Box.GENERATE_NORMALS,stripeAppear));
            roadGeometry.addChild(strLong2broken);
            
            if (type == Constant.ROADSLEFT)
               modelTransform.set(new Vector3f((lengthOfBS+subroad.width)/2.0f-0.15f, 0.0f,(-width/2.0f)+0.15f));
            else
               modelTransform.set(new Vector3f((lengthOfBS+subroad.width)/2.0f-0.15f, 0.0f,(width/2.0f)-0.15f));
            TransformGroup strLong3broken = new TransformGroup(modelTransform);
            strLong3broken.addChild(new Box(lengthOfBS/2.0f,0.01f,0.05f,Box.GENERATE_NORMALS,stripeAppear));
            roadGeometry.addChild(strLong3broken);
         }
         
      } else if (type == Constant.TURNLEFT) {
         // A road with a turn left consists of two subroads: north and west, and the turn itself.
         float sharpness = 3.0f;
         
         Turn turnedRoadPart = new Turn();
         turnedRoadPart.sharpness = sharpness;
         turnedRoadPart.width = this.width;
         turnedRoadPart.textureOn = this.textureOn;
         turnedRoadPart.createGeometry();
         roadGeometry.addChild(turnedRoadPart.getGeometry()); 
         
         // Add a north and westroad.
         Road northroad = new Road();
         Road westroad = new Road();
         
         // Set the properties
         if (!westroadSpecified) {
            westroad.textureOn = this.textureOn;
            westroad.sideStripes = this.sideStripes;
            westroad.middleStripes = this.middleStripes;
            westroad.width = this.width;
            westroad.length = this.length;
         } else
            westroad = wRoad;
         if (!northroadSpecified) {
            northroad.textureOn = this.textureOn;
            northroad.sideStripes = this.sideStripes;
            northroad.middleStripes = this.middleStripes;
            northroad.width = this.width;
            northroad.length = this.length;
         } else
            northroad = nRoad;
         
         northroad.createGeometry();
         westroad.createGeometry();
         
         Transform3D translate = new Transform3D();
         Transform3D rotate = new Transform3D();
         rotate.rotY(Math.PI/2.0d);
         translate.set(new Vector3f(0.0f, 0.0f, -( sharpness + width/2.0f + northroad.length/2.0f)));
         translate.mul(rotate);
         TransformGroup north = new TransformGroup(translate);
         north.addChild(northroad.getGeometry());
         roadGeometry.addChild(north);
         translate.set(new Vector3f(-( sharpness + width/2.0f + westroad.length/2.0f), 0.0f, 0.0f));
         TransformGroup west = new TransformGroup(translate);
         west.addChild(westroad.getGeometry());
         roadGeometry.addChild(west);
         
         // Create sidestripes, if necessary
         if (sideStripes) {
            // The sidestripes are actually two turns!
            Turn littleStripe = new Turn();
            littleStripe.sharpness = sharpness + 0.05f;
            littleStripe.width = 0.1f;
            littleStripe.textureOn = false;
            littleStripe.stripe = true;
            littleStripe.createGeometry();
            TransformGroup ls = littleStripe.getGeometry();
            translate.set(new Vector3f(-(this.width/2.0f - 0.15f), 0.01f, -(this.width/2.0f - 0.15f)));
            TransformGroup littleStripeG = new TransformGroup(translate);
            littleStripeG.addChild(littleStripe.getGeometry());
            roadGeometry.addChild(littleStripeG);
            
            Turn largeStripe = new Turn();
            largeStripe.sharpness = sharpness + this.width - 0.25f;
            largeStripe.width = 0.1f;
            largeStripe.textureOn = false;
            largeStripe.stripe = true;
            largeStripe.createGeometry();
            translate.set(new Vector3f(this.width/2.0f - 0.15f, 0.01f, this.width/2.0f - 0.15f));
            TransformGroup largeStripeG = new TransformGroup(translate);
            largeStripeG.addChild(largeStripe.getGeometry());
            roadGeometry.addChild(largeStripeG);
         }
         
         // Create middlestripes, if necessary
         if (middleStripes) {
            double amountOfStripes = Math.PI/2.0d * (double)(sharpness+this.width/2.0f)/1.5d;
            double angle = 1.5d/(double)(sharpness+this.width/2.0f);
            translate.set(new Vector3f(-(sharpness+ width/2.0f), 0.0f, -(sharpness +width/2.0f)));
            TransformGroup totalStr = new TransformGroup(translate);
            roadGeometry.addChild(totalStr);
            
            for (double i=angle/2.0d; i<Math.PI/2.0f; i += angle) {
               rotate.rotY(i);
               translate.set(new Vector3f(0.0f, 0.0f, sharpness+this.width/2.0f));
               rotate.mul(translate);
               TransformGroup tStripe = new TransformGroup(rotate);
               tStripe.addChild(new Box(0.2f,0.01f,0.05f,Box.GENERATE_NORMALS,stripeAppear));
               totalStr.addChild(tStripe);
            }
         }
        
      } else if (type == Constant.TURNRIGHT) {
         // A road with a turn right consists of two subroads: south and west, and the turn itself.
         float sharpness = 3.0f;
         
         Turn turnedRoadPart = new Turn();
         turnedRoadPart.sharpness = sharpness;
         turnedRoadPart.width = this.width;
         turnedRoadPart.textureOn = this.textureOn;
         turnedRoadPart.left = false;
         turnedRoadPart.createGeometry();
         roadGeometry.addChild(turnedRoadPart.getGeometry()); 
         
         // Add a north and westroad.
         Road southroad = new Road();
         Road westroad = new Road();
         
         // Set the properties
         if (!westroadSpecified) {
            westroad.textureOn = this.textureOn;
            westroad.sideStripes = this.sideStripes;
            westroad.middleStripes = this.middleStripes;
            westroad.width = this.width;
            westroad.length = this.length;
         } else
            westroad = wRoad;
         if (!southroadSpecified) {
            southroad.textureOn = this.textureOn;
            southroad.sideStripes = this.sideStripes;
            southroad.middleStripes = this.middleStripes;
            southroad.width = this.width;
            southroad.length = this.length;
         } else
            southroad = sRoad;
         
         southroad.createGeometry();
         westroad.createGeometry();
         
         Transform3D translate = new Transform3D();
         Transform3D rotate = new Transform3D();
         rotate.rotY(-Math.PI/2.0d);
         translate.set(new Vector3f(0.0f, 0.0f, sharpness + width/2.0f + southroad.length/2.0f));
         translate.mul(rotate);
         TransformGroup south = new TransformGroup(translate);
         south.addChild(southroad.getGeometry());
         roadGeometry.addChild(south);
         translate.set(new Vector3f(-( sharpness + width/2.0f + westroad.length/2.0f), 0.0f, 0.0f));
         TransformGroup west = new TransformGroup(translate);
         west.addChild(westroad.getGeometry());
         roadGeometry.addChild(west);
         
         // Create sidestripes, if necessary
         if (sideStripes) {
            // The sidestripes are actually two turns!
            Turn littleStripe = new Turn();
            littleStripe.sharpness = sharpness + 0.05f;
            littleStripe.width = 0.1f;
            littleStripe.textureOn = false;
            littleStripe.stripe = true;
            littleStripe.left = false;
            littleStripe.createGeometry();
            TransformGroup ls = littleStripe.getGeometry();
            translate.set(new Vector3f(-(this.width/2.0f - 0.15f), 0.01f, this.width/2.0f - 0.15f));
            TransformGroup littleStripeG = new TransformGroup(translate);
            littleStripeG.addChild(littleStripe.getGeometry());
            roadGeometry.addChild(littleStripeG);
            
            Turn largeStripe = new Turn();
            largeStripe.sharpness = sharpness + this.width - 0.25f;
            largeStripe.width = 0.1f;
            largeStripe.textureOn = false;
            largeStripe.stripe = true;
            largeStripe.left = false;
            largeStripe.createGeometry();
            translate.set(new Vector3f(this.width/2.0f - 0.15f, 0.01f, -(this.width/2.0f - 0.15f)));
            TransformGroup largeStripeG = new TransformGroup(translate);
            largeStripeG.addChild(largeStripe.getGeometry());
            roadGeometry.addChild(largeStripeG);
         }
         
         // Create middlestripes, if necessary
         if (middleStripes) {
            double amountOfStripes = Math.PI/2.0d * (double)(sharpness+this.width/2.0f)/1.5d;
            double angle = 1.5d/(double)(sharpness+this.width/2.0f);
            translate.set(new Vector3f(-(sharpness+ width/2.0f), 0.0f, sharpness +width/2.0f));
            TransformGroup totalStr = new TransformGroup(translate);
            roadGeometry.addChild(totalStr);
            
            for (double i=Math.PI-angle/2.0d; i>Math.PI/2.0f; i -= angle) {
               rotate.rotY(i);
               translate.set(new Vector3f(0.0f, 0.0f, sharpness+this.width/2.0f));
               rotate.mul(translate);
               TransformGroup tStripe = new TransformGroup(rotate);
               tStripe.addChild(new Box(0.2f,0.01f,0.05f,Box.GENERATE_NORMALS,stripeAppear));
               totalStr.addChild(tStripe);
            }
         }

      }
   }
   
   // Returns the geometry of the Road object
   public TransformGroup getGeometry() {
      return roadGeometry;
   } 
   
   // Methods for recurrent adding of subroads
   
   // Specifies subr as subroad of this Road
   public void specifySubroad(Road subr) { 
      subRoad = subr;
      subroadSpecified = true;
   }
   
   // Specifies nr as north road of this Road
   public void specifyNorthroad(Road nr) { 
      nRoad = nr;
      northroadSpecified = true;
   }
   
   // Specifies sr as south road of this Road
   public void specifySouthroad(Road sr) { 
      sRoad = sr;
      southroadSpecified = true;
   }
   
   // Specifies er as east road of this Road
   public void specifyEastroad(Road er) { 
      eRoad = er;
      eastroadSpecified = true;
   }
   
   // Specifies wr as west road of this Road
   public void specifyWestroad(Road wr) { 
      wRoad = wr;
      westroadSpecified = true;
   }   
   
   // Returns the collision radius
   // === NOTE: this is a ridiculous value, because a collision with a 
   //           road is impossible! ===
   public float getCollisionRadius() {
      return -1.0f;
   }
   
   // Returns the id (also a ridiculous value, because there is only 1 road)
   public String getId() {
      return "";
   }
   
   // Returns the x position
   public float getXPos() {
      return 0.0f;
   }
   
   // Returns the y position
   public float getYPos() {
      return 0.0f;
   }
}