// This creates a standard SimpleCar object.

// 	Name: SimpleCar (class)
//	Author: Arjan Egges
//	Version: 1.0


package  se.lth.cs.carsimgui;

import com.sun.j3d.utils.behaviors.interpolators.*;
import java.util.Date;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;

class SimpleCar implements DynamicAccidentObject {

   // Property variables
   public float length;
   public float width;
   public float height;
   public float wheelRadius;
   public float wheelWidth;
   public String id;

   // Private members
   private TransformGroup movingCar;
   private RotPosScaleTCBSplinePathInterpolator rotPosInt;
   private Trajectory traject;
   private Alpha alpha;
   private long timeStamp;
   private long startTime;  

   // Constructor
   public SimpleCar() {
      length = 3.5f;
      width = 1.5f;
      height = 1.0f;
      wheelRadius = 0.3f;
      wheelWidth = 0.25f;
      id = "";
      
      // Dynamic properties
      traject = new Trajectory();
      timeStamp = new Date().getTime(); 
   }
 
   // Creates the geometry of the simple car  
   public void createGeometry() {
      // Dynamic property variables
      float startPosX = traject.get(0).x;
      float startPosY = traject.get(0).y;
      
      Transform3D translate = new Transform3D();
      translate.set(new Vector3f(0.0f,0.0f,0.0f));
      TransformGroup carGeo = new TransformGroup(translate);
      
      // A simple car consists of a box and four cylinders
      Appearance cabAppear = new Appearance();
      Appearance trailer1Appear = new Appearance();
      Appearance trailer2Appear = new Appearance();
      Appearance baseAppear = new Appearance();
      //Color3f eColor    = new Color3f(0.0f, 0.0f, 0.0f);
      //Color3f sColor    = new Color3f(1.0f, 1.0f, 1.0f);
      //Color3f oColor    = new Color3f(0.9f, 0.4f, 0.6f);
      
      Color3f eColor    = new Color3f(0.0f, 0.0f, 0.0f);
      Color3f e2Color   = new Color3f(0.2f, 0.2f, 0.25f);
      Color3f sColor    = new Color3f(1.0f, 1.0f, 1.0f);
      Color3f oColor    = new Color3f(0.9f, 0.4f, 0.6f);
      Color3f oCab      = new Color3f(0.0f, 0.0f, 0.3f);
      Color3f oTrailer1 = new Color3f(1.0f, 0.1f, 0.1f);
      Color3f oTrailer2 = new Color3f(0.98f, 0.95f, 1.0f);
      Color3f oBase     = new Color3f(0.2f, 0.2f, 0.2f);
      
      Material cabMat = new Material(oCab, e2Color, oCab, sColor, 100.0f);
      cabMat.setLightingEnable(true);
      cabAppear.setMaterial(cabMat);
      Material trailer1Mat = new Material(oTrailer1, eColor, oTrailer1, sColor, 100.0f);
      trailer1Mat.setLightingEnable(true);
      trailer1Appear.setMaterial(trailer1Mat);
      Material trailer2Mat = new Material(oTrailer2, e2Color, oTrailer2, sColor, 100.0f);
      trailer2Mat.setLightingEnable(true);
      trailer2Appear.setMaterial(trailer2Mat);
      Material baseMat = new Material(oBase, eColor, oBase, sColor, 100.0f);
      baseMat.setLightingEnable(true);
      baseAppear.setMaterial(baseMat);
            
      Appearance wheelAppear = new Appearance();
      eColor    = new Color3f(0.0f, 0.0f, 0.0f);
      sColor    = new Color3f(1.0f, 1.0f, 1.0f);
      oColor    = new Color3f(0.5f, 0.5f, 0.5f);
      
      Material wheelMat = new Material(oColor, eColor, oColor, sColor, 100.0f);
      wheelMat.setLightingEnable(true);
      wheelAppear.setMaterial(wheelMat);
      
      // Add four cylinders and translate them
      Transform3D rotate = new Transform3D();
      rotate.rotX(Math.PI/2.0f);
      TransformGroup wheelTrans;
      
      // the wheels
      // tractor
      // 1st R
      translate.set(new Vector3f((length/2.0f)*0.93f-wheelRadius, wheelRadius, width/2.0f-wheelWidth-0.02f));
      translate.mul(rotate);
      wheelTrans = new TransformGroup(translate);
      wheelTrans.addChild(new Cylinder(wheelRadius, wheelWidth,wheelAppear));
      carGeo.addChild(wheelTrans);
      // 1st L
      translate.set(new Vector3f((length/2.0f)*0.93f-wheelRadius, wheelRadius, -width/2.0f+wheelWidth+0.02f));
      translate.mul(rotate);
      wheelTrans = new TransformGroup(translate);
      wheelTrans.addChild(new Cylinder(wheelRadius, wheelWidth,wheelAppear));
      carGeo.addChild(wheelTrans);
      // 2nd R
      translate.set(new Vector3f((length/2.0f)*0.55f-wheelRadius, wheelRadius, width/2.0f-wheelWidth-0.02f));
      translate.mul(rotate);
      wheelTrans = new TransformGroup(translate);
      wheelTrans.addChild(new Cylinder(wheelRadius, wheelWidth,wheelAppear));
      carGeo.addChild(wheelTrans);
      // 2nd L
      translate.set(new Vector3f((length/2.0f)*0.55f-wheelRadius, wheelRadius, -width/2.0f+wheelWidth+0.02f));
      translate.mul(rotate);
      wheelTrans = new TransformGroup(translate);
      wheelTrans.addChild(new Cylinder(wheelRadius, wheelWidth,wheelAppear));
      carGeo.addChild(wheelTrans);
      // 3rd R
      translate.set(new Vector3f((length/2.0f)*0.33f-wheelRadius, wheelRadius, width/2.0f-wheelWidth-0.02f));
      translate.mul(rotate);
      wheelTrans = new TransformGroup(translate);
      wheelTrans.addChild(new Cylinder(wheelRadius, wheelWidth,wheelAppear));
      carGeo.addChild(wheelTrans);
      // 3rd L
      translate.set(new Vector3f((length/2.0f)*0.33f-wheelRadius, wheelRadius, -width/2.0f+wheelWidth+0.02f));
      translate.mul(rotate);
      wheelTrans = new TransformGroup(translate);
      wheelTrans.addChild(new Cylinder(wheelRadius, wheelWidth,wheelAppear));
      carGeo.addChild(wheelTrans);
      
      // trailer
      // 1st R
      translate.set(new Vector3f(-(length/2.0f)*0.68f+wheelRadius, wheelRadius, width/2.0f-wheelWidth-0.02f));
      translate.mul(rotate);
      wheelTrans = new TransformGroup(translate);
      wheelTrans.addChild(new Cylinder(wheelRadius, wheelWidth,wheelAppear));
      carGeo.addChild(wheelTrans);
      // 1st L
      translate.set(new Vector3f(-(length/2.0f)*0.68f+wheelRadius, wheelRadius, -width/2.0f+wheelWidth+0.02f));
      translate.mul(rotate);
      wheelTrans = new TransformGroup(translate);
      wheelTrans.addChild(new Cylinder(wheelRadius, wheelWidth,wheelAppear));
      carGeo.addChild(wheelTrans);
      // 2nd R
      translate.set(new Vector3f(-(length/2.0f)*0.9f+wheelRadius, wheelRadius, width/2.0f-wheelWidth-0.02f));
      translate.mul(rotate);
      wheelTrans = new TransformGroup(translate);
      wheelTrans.addChild(new Cylinder(wheelRadius, wheelWidth,wheelAppear));
      carGeo.addChild(wheelTrans);
      // 2nd L
      translate.set(new Vector3f(-(length/2.0f)*0.9f+wheelRadius, wheelRadius, -width/2.0f+wheelWidth+0.02f));
      translate.mul(rotate);
      wheelTrans = new TransformGroup(translate);
      wheelTrans.addChild(new Cylinder(wheelRadius, wheelWidth,wheelAppear));
      carGeo.addChild(wheelTrans);
      
      // Commented code is for defining an additional cube to distinguish the
      // front of the vehicle from the back
      
      // translate.set(new Vector3f(2.0f, 1.0f, 1.0f));
      // TransformGroup fr = new TransformGroup(translate);
      // fr.addChild(new Box(0.4f, 0.4f, 0.4f, Box.GENERATE_NORMALS,carAppear));
      // carGeo.addChild(fr);
      
      translate = new Transform3D();
      TransformGroup carTrans;
      
      // Box mainCar = new Box(length/2.0f,height/2.0f,width/2.0f,Box.GENERATE_NORMALS,carAppear);
      // translate.set(new Vector3f(0.0f, (height/2.0f)+wheelRadius*1.5f, 0.0f));
      // carTrans = new TransformGroup(translate);
      // carTrans.addChild(mainCar);
      // carGeo.addChild(carTrans);
      
      // the base plate of the tractor and the trailer
      Box truckBase = new Box(length/2.0f,height*0.05f,width/2.0f,Box.GENERATE_NORMALS,baseAppear);
      translate.set(new Vector3f(0.0f, (height*0.05f)+wheelRadius*1.5f, 0.0f));
      carTrans = new TransformGroup(translate);
      carTrans.addChild(truckBase);
      carGeo.addChild(carTrans);
      
      // the red-white-red trailer
      Box trailer1 = new Box(length*0.35f,height*0.05f,width/2.0f,Box.GENERATE_NORMALS,trailer1Appear);
      translate.set(new Vector3f((-length*0.15f), (height*0.15f)+wheelRadius*1.5f, 0.0f));
      carTrans = new TransformGroup(translate);
      carTrans.addChild(trailer1);
      carGeo.addChild(carTrans);
      
      Box trailer2 = new Box(length*0.35f,height*0.05f,width/2.0f,Box.GENERATE_NORMALS,trailer2Appear);
      translate.set(new Vector3f((-length*0.15f), (height*0.25f)+wheelRadius*1.5f, 0.0f));
      carTrans = new TransformGroup(translate);
      carTrans.addChild(trailer2);
      carGeo.addChild(carTrans);
      
      Box trailer3 = new Box(length*0.35f,height*0.05f,width/2.0f,Box.GENERATE_NORMALS,trailer1Appear);
      translate.set(new Vector3f((-length*0.15f), (height*0.35f)+wheelRadius*1.5f, 0.0f));
      carTrans = new TransformGroup(translate);
      carTrans.addChild(trailer3);
      carGeo.addChild(carTrans);
      
      Box trailer4 = new Box(length*0.35f,height*0.05f,width/2.0f,Box.GENERATE_NORMALS,trailer2Appear);
      translate.set(new Vector3f((-length*0.15f), (height*0.45f)+wheelRadius*1.5f, 0.0f));
      carTrans = new TransformGroup(translate);
      carTrans.addChild(trailer4);
      carGeo.addChild(carTrans);
      
      Box trailer5 = new Box(length*0.35f,height*0.05f,width/2.0f,Box.GENERATE_NORMALS,trailer1Appear);
      translate.set(new Vector3f((-length*0.15f), (height*0.55f)+wheelRadius*1.5f, 0.0f));
      carTrans = new TransformGroup(translate);
      carTrans.addChild(trailer5);
      carGeo.addChild(carTrans);
      
      Box trailer6 = new Box(length*0.35f,height*0.05f,width/2.0f,Box.GENERATE_NORMALS,trailer2Appear);
      translate.set(new Vector3f((-length*0.15f), (height*0.65f)+wheelRadius*1.5f, 0.0f));
      carTrans = new TransformGroup(translate);
      carTrans.addChild(trailer6);
      carGeo.addChild(carTrans);
      
      Box trailer7 = new Box(length*0.35f,height*0.05f,width/2.0f,Box.GENERATE_NORMALS,trailer1Appear);
      translate.set(new Vector3f((-length*0.15f), (height*0.75f)+wheelRadius*1.5f, 0.0f));
      carTrans = new TransformGroup(translate);
      carTrans.addChild(trailer7);
      carGeo.addChild(carTrans);
      
      Box trailer8 = new Box(length*0.35f,height*0.05f,width/2.0f,Box.GENERATE_NORMALS,trailer2Appear);
      translate.set(new Vector3f((-length*0.15f), (height*0.85f)+wheelRadius*1.5f, 0.0f));
      carTrans = new TransformGroup(translate);
      carTrans.addChild(trailer8);
      carGeo.addChild(carTrans);
      
      Box trailer9 = new Box(length*0.35f,height*0.05f,width/2.0f,Box.GENERATE_NORMALS,trailer1Appear);
      translate.set(new Vector3f((-length*0.15f), (height*0.95f)+wheelRadius*1.5f, 0.0f));
      carTrans = new TransformGroup(translate);
      carTrans.addChild(trailer9);
      carGeo.addChild(carTrans);
      
      // the motor
      Box motor = new Box(length*0.05f,height*0.2f,width*0.3f,Box.GENERATE_NORMALS,cabAppear);
      translate.set(new Vector3f((length*0.45f), (height*0.25f)+wheelRadius*1.5f, 0.0f));
      carTrans = new TransformGroup(translate);
      carTrans.addChild(motor);
      carGeo.addChild(carTrans);
      
      // the driving cab
      Box drivingCab = new Box(length*0.075f,height*0.33f,width/2.0f,Box.GENERATE_NORMALS,cabAppear);
      translate.set(new Vector3f((length*0.325f), (height*0.38f)+wheelRadius*1.5f, 0.0f));
      carTrans = new TransformGroup(translate);
      carTrans.addChild(drivingCab);
      carGeo.addChild(carTrans);
      
      
      // The next part of the code specifies the behavior of the car. This means traveling along the
      // road between two given points in the XZ plane.
           
      translate.set(new Vector3f(startPosX, 0.0f, startPosY));
      rotate.rotY(traject.getInitAngle());
      translate.mul(rotate);
      movingCar = new TransformGroup(translate);
      
      movingCar.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
      
      // Create behavior
      TrajectoryBehavior tb = new TrajectoryBehavior(this);
      tb.createBehavior();
      alpha = tb.alpha;
      
      rotPosInt = tb.getBehavior();
      rotPosInt.setEnable(false);
      movingCar.addChild(carGeo);
      movingCar.addChild(rotPosInt);
   
   }
   
   // Returns the geometry of the simple car
   public TransformGroup getGeometry() {
      return movingCar;
   }
   
   // Sets the trajectory of the SimpleCar object
   public void setTrajectory(Trajectory t) {
      traject = t;
   }
   
   // Returns the trajectory of the SimpleCar object
   public Trajectory getTrajectory() {
      return traject;
   }
   
   // Some methods for handling dynamics 
   
   // Start animation
   public void play() {
      startTime = new Date().getTime()+500;
      alpha.setStartTime(startTime);
      rotPosInt.setEnable(true);    
   }
   
   // Pause animation
   public void pause() {
      timeStamp = new Date().getTime() - startTime;
      rotPosInt.setEnable(false);
   }
   
   // Resume animation
   public void resume() {
      startTime = new Date().getTime() - timeStamp;
      alpha.setStartTime(startTime);
      rotPosInt.setEnable(true);
   }
   
   // Stop animation
   public void stop() {
      rotPosInt.setEnable(false);
   }  
   
   // Returns the id
   public String getId() {
      return id;
   }
   
   // Set the id to s
   public void setId(String s) {
      id = s;
   }
   
   // Returns the length of the vehicle
   public float getLength() {
      return length;
   }
   
   // Returns the width of the vehicle
   public float getWidth() {
      return width;
   }
   
   // Returns the height of the vehicle
   public float getHeight() {
      return height;
   }
   

   
}