// Class for parsing a simple String

//	Name: StringParser (class)
//	Author: Arjan Egges
//	Version 1.0

package  se.lth.cs.carsimgui;

class StringParser {

   // Private members.
   private String parseString;
   private int index;
   private int line;
   private String tokenList = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_0123456789-.";
   private String singleTokenList = "[];=,()";
   private char commentchar = '/';
   
   // Constructor.
   public StringParser(String str) {
      parseString = str;
      index = 0;
      line = 1;
      skipGarbage();
   }
   
   // Delivers the next token. If the end of the string is reached, "" is returned.
   public String nextToken() {
      String token = new String("");
      
      if (index < parseString.length() && isInSingleTokenList(parseString.charAt(index))) {
         token = token + String.valueOf(parseString.charAt(index));
         index++;
      } else   
         while (index < parseString.length() && isInTokenList(parseString.charAt(index))) {
            token = token + String.valueOf(parseString.charAt(index));
            index++;
         }
      skipGarbage();
      return token;
   }
   
   // Skips garbage
   private void skipGarbage() {  
      String garbage = "";      
      while (index < parseString.length() && !isInTokenList(parseString.charAt(index))
                            && !isInSingleTokenList(parseString.charAt(index))) {
         if (parseString.charAt(index) == '\n')
            line++;
         if (parseString.charAt(index) == commentchar) 
            garbage = garbage + parseString.charAt(index);
         index++;
         if (garbage.equals("//")) {
            while (index < parseString.length() && parseString.charAt(index) != '\n') {
               garbage = garbage + parseString.charAt(index);
               index++;  
            }
            skipGarbage();
         }
      } 
   }
   
   // Private methods for checking if a character belongs to a token.
   private boolean isInTokenList(char c) {
      int i = 0;
      while (i<tokenList.length() && c != tokenList.charAt(i))
         i++;
      return (i < tokenList.length());
   } 
   
   private boolean isInSingleTokenList(char c) {
      int i = 0;
      while (i<singleTokenList.length() && c != singleTokenList.charAt(i))
         i++;
      return (i < singleTokenList.length());
   
   }
   
   // Returns the character at which the index points to.
   public char currChar() {
      if (index < parseString.length())
         return parseString.charAt(index);
      else
         return ' ';
   }
   
   // Returns the current line
   public int currLine() {
      return line;
   }
   
   // Returns true if the end of the parsing string has been reached, false otherwise.
   public boolean endOfString() {
      return (index >= parseString.length());
   }  
}