// This class provides method for creating a (staitc) traffic light
// with a certain color

// 	Name: TrafficLight (class)
//	Author: Arjan Egges
//	Version: 1.0


package  se.lth.cs.carsimgui;

import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;

class TrafficLight implements AccidentObject {

   public float xPos;
   public float yPos;
   public String id;
   public int color;
   
   // Private members
   TransformGroup lightGeo;
   
   // Constructor
   public TrafficLight() {
      xPos = 0.0f;
      yPos = 0.0f;
      id = "red light";
      color = Constant.RED;
   }
   
   // Creates the geometry of the Traffic Light
   public void createGeometry() {
      Transform3D translate = new Transform3D();
      translate.set(new Vector3f(xPos,0.0f,yPos));
      lightGeo = new TransformGroup(translate);
      
      // Create a cylinder
      Appearance cylAppear = new Appearance();
      Color3f eColor    = new Color3f(0.0f, 0.0f, 0.0f);
      Color3f sColor    = new Color3f(1.0f, 1.0f, 1.0f);
      Color3f oColor    = new Color3f(0.0f, 0.0f, 0.0f);
      
      Material cylMat = new Material(oColor, eColor, oColor, sColor, 100.0f);
      cylMat.setLightingEnable(true);
      cylAppear.setMaterial(cylMat);
      translate.set(new Vector3f(0.0f, 2.5f, 0.0f));
      TransformGroup cylTrans = new TransformGroup(translate);
      lightGeo.addChild(cylTrans);
      cylTrans.addChild(new Cylinder(0.08f,5.0f,cylAppear));
      
      // Create a box
      Appearance boxAppear = new Appearance();
      eColor    = new Color3f(0.0f, 0.0f, 0.0f);
      sColor    = new Color3f(1.0f, 1.0f, 1.0f);
      oColor    = new Color3f(0.0f, 0.0f, 0.0f);
      
      Material boxMat = new Material(oColor, eColor, oColor, sColor, 100.0f);
      boxMat.setLightingEnable(true);
      boxAppear.setMaterial(boxMat);
      translate.set(new Vector3f(0.0f, 5.4f, 0.0f));
      TransformGroup boxTrans = new TransformGroup(translate);
      lightGeo.addChild(boxTrans);
      boxTrans.addChild(new Box(0.4f,1.1f,0.4f,Box.GENERATE_NORMALS,boxAppear));
      
      // Add three spheres for the lights
      
      // Set the translate groups
      translate.set(new Vector3f(0.0f,6.0f,0.0f));
      TransformGroup redLightTrans = new TransformGroup(translate);
      lightGeo.addChild(redLightTrans);
      translate.set(new Vector3f(0.0f,5.4f,0.0f));
      TransformGroup orangeLightTrans = new TransformGroup(translate);
      lightGeo.addChild(orangeLightTrans);
      translate.set(new Vector3f(0.0f,4.8f,0.0f));
      TransformGroup greenLightTrans = new TransformGroup(translate);
      lightGeo.addChild(greenLightTrans);
      
      // Create the appearances
      // Passive light (inactive)
      Appearance passiveLightAppear = new Appearance();
      eColor    = new Color3f(0.0f, 0.0f, 0.0f);
      sColor    = new Color3f(1.0f, 1.0f, 1.0f);
      oColor    = new Color3f(0.2f, 0.2f, 0.2f);
      Material passiveLightMat = new Material(oColor, eColor, oColor, sColor, 100.0f);
      passiveLightMat.setLightingEnable(true);
      passiveLightAppear.setMaterial(passiveLightMat);
      // Red light
      oColor 	= new Color3f(1.0f, 0.0f, 0.0f);
      eColor    = new Color3f(0.9f, 0.1f, 0.1f);
      Material redLightMat = new Material(oColor, eColor, oColor, sColor, 100.0f);
      redLightMat.setLightingEnable(true);
      Appearance redLightAppear = new Appearance();
      redLightAppear.setMaterial(redLightMat);
      // Green light
      oColor 	= new Color3f(0.0f, 1.0f, 0.0f);
      eColor    = new Color3f(0.1f, 0.9f, 0.1f);
      Material greenLightMat = new Material(oColor, eColor, oColor, sColor, 100.0f);
      greenLightMat.setLightingEnable(true);
      Appearance greenLightAppear = new Appearance();
      greenLightAppear.setMaterial(greenLightMat);
      // Orange light
      oColor 	= new Color3f(1.0f, 1.0f, 0.0f);
      eColor    = new Color3f(0.7f, 0.7f, 0.1f);
      Material orangeLightMat = new Material(oColor, eColor, oColor, sColor, 100.0f);
      orangeLightMat.setLightingEnable(true);
      Appearance orangeLightAppear = new Appearance();
      orangeLightAppear.setMaterial(orangeLightMat);
      
      // Upper light
      if (color == Constant.RED)
         redLightTrans.addChild(new Sphere(0.47f,redLightAppear));
      else 
         redLightTrans.addChild(new Sphere(0.47f,passiveLightAppear));
      
      
      // Middle light
      if (color == Constant.ORANGE)
         orangeLightTrans.addChild(new Sphere(0.47f,orangeLightAppear));
      else 
         orangeLightTrans.addChild(new Sphere(0.47f,passiveLightAppear));
      
      // Lower light
      if (color == Constant.GREEN)
         greenLightTrans.addChild(new Sphere(0.47f,greenLightAppear));
      else 
         greenLightTrans.addChild(new Sphere(0.47f,passiveLightAppear));
   
   }
   
   // Returns the geometry of the Traffic Light
   public TransformGroup getGeometry() {
      return lightGeo;
   }
   
   // Returns the collision radius
   public float getCollisionRadius() {
      return 1.0f;
   }
   
   // Returns the id
   public String getId() {
      return id;
   }
   
   // Returns the x position
   public float getXPos() {
      return xPos;
   }
   
   // Returns the y position
   public float getYPos() {
      return yPos;
   }
   
}