// This creates a turn (for usage in constructing a road object).

// 	Name: Turn (class)
//	Author: Arjan Egges
//	Version: 1.0


package  se.lth.cs.carsimgui;

import com.sun.j3d.utils.geometry.*;
//import NewTextureLoader;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.image.TextureLoader;


class Turn implements AccidentObject {

   // Property variables
   public double alpha;

  public float width;

  public float sharpness;

  public int precision;

  public boolean left;

  public boolean stripe;

   public boolean textureOn;
   
   // Private members
   TransformGroup turnGeo;
   
   // Constructor
   public Turn() {
      alpha = Math.PI/2.0d;
      width = 6.0f;
      sharpness = 0.5f;
      precision = 15;
      left = true;
      stripe = false;
      
      textureOn = true;
   }
   
   // Creates the geometry of the Turn
   public void createGeometry() {
      Transform3D rotate = new Transform3D();
      Transform3D translate = new Transform3D();
      rotate.rotX(-Math.PI/2.0d);
      if (left) {
         Transform3D rotate2 = new Transform3D();
         rotate2.rotY(-Math.PI/2.0d); 
            
         translate.set(new Vector3f(-(sharpness + width/2.0f), 0.0f, -(sharpness + width/2.0f)));
         rotate2.mul(rotate);
         translate.mul(rotate2);
      } else {         
         
         translate.set(new Vector3f(-(sharpness + width/2.0f), 0.0f, sharpness + width/2.0f));
         translate.mul(rotate);
         
      } 
      turnGeo = new TransformGroup(translate);
      
      // Now create the turn as a Geometric object
      float[] coordinateData = null;
      coordinateData = createCoordinateData();
      int[] stripCount = {2*(precision+1)+1};
        
      GeometryInfo gi = new GeometryInfo(GeometryInfo.POLYGON_ARRAY);
      gi.setCoordinates(coordinateData);
      gi.setStripCounts(stripCount);
        
      Triangulator tr = new Triangulator();
      tr.triangulate(gi);
      gi.recomputeIndices();

      NormalGenerator ng = new NormalGenerator();
      ng.generateNormals(gi);
      gi.recomputeIndices();

      Stripifier st = new Stripifier();
      st.stripify(gi);
      gi.recomputeIndices();

      Shape3D part = new Shape3D();
      part.setGeometry(gi.getGeometryArray());
      
      // Add the right appearance
      
      if (textureOn)
         part.setAppearance(createTexturedAppearance("images/rock033_3.jpg"));
      else
         part.setAppearance(createMaterialAppearance());  
      
      turnGeo.addChild(part);       
         
   }
   
   // Returns the geometry of the turn
   public TransformGroup getGeometry() {
      return turnGeo;
   }
   
   // Private methods
   
   private float[] createCoordinateData() {
       float[] data = new float[(2*(precision+1)+1)*3];
       int i = 0;
       
       data[i++] = sharpness; data[i++] = 0.0f; data[i++] = 0.0f; // 0
       
       double partalpha = alpha/(double) precision;
       for (double tau = 0; tau<=alpha; tau += alpha/(double) precision) {
          float xVal = (float) Math.cos(tau)*(sharpness+width);
          float yVal = (float) Math.sin(tau)*(sharpness+width);
          
          data[i++] = xVal; data[i++] = yVal; data[i++] = 0.0f;
       }
       
       for (double tau = alpha; tau>=0; tau -= alpha/(double) precision) {
          float xVal = (float) Math.cos(tau)*sharpness;
          float yVal = (float) Math.sin(tau)*sharpness;
          
          data[i++] = xVal; data[i++] = yVal; data[i++] = 0.0f;
       }
       
       return data;
    } 
    
    // Create the material appearance    
    private Appearance createMaterialAppearance(){

      Appearance rockAppear = new Appearance();

      if (!stripe) {          
          Color3f eColor    = new Color3f(0.0f, 0.0f, 0.0f);
          Color3f sColor    = new Color3f(1.0f, 1.0f, 1.0f);
          Color3f oColor    = new Color3f(0.8f, 0.8f, 0.8f);
      
          Material rockMat = new Material(oColor, eColor, oColor, sColor, 100.0f);
          rockMat.setLightingEnable(true);
          rockAppear.setMaterial(rockMat);
       } else {
          Color3f streColor    = new Color3f(0.8f, 0.8f, 0.8f);
          Color3f strsColor    = new Color3f(1.0f, 1.0f, 1.0f);
          Color3f stroColor    = new Color3f(1.0f, 1.0f, 1.0f);
          Material stripeMat = new Material(stroColor, streColor, stroColor, strsColor, 100.0f);
          stripeMat.setLightingEnable(true);
          rockAppear.setMaterial(stripeMat);
       }
       return rockAppear;
    }

    // Create the textured appearance
    private Appearance createTexturedAppearance(String filename) {

      Appearance appear = new Appearance();

      Messenger.message("Attempt to load file: "+filename+"...");
      TextureLoader loader = new NewTextureLoader(filename);
      ImageComponent2D image = loader.getImage();

      if(image == null) {
         Messenger.message("load failed for texture: "+filename+"\n");
      }
else {
         Messenger.message("succeeded\n");
      }
   
      // can't use parameterless constuctor
      Texture2D texture = new Texture2D(Texture.BASE_LEVEL, Texture.RGB,
image.getWidth(), image.getHeight());
      texture.setImage(0, image);
      texture.setEnable(true);
      texture.setMagFilter(Texture.NICEST);
      appear.setTexture(texture);
      
      // Generate the Texture Coordinates
      TexCoordGeneration tc = new TexCoordGeneration(TexCoordGeneration.OBJECT_LINEAR,
                                    TexCoordGeneration.TEXTURE_COORDINATE_2);
      tc.setEnable(true);
      appear.setTexCoordGeneration(tc);
      return appear;
    }

   // Returns the collision radius
   // === NOTE: this is a ridiculous value, because a collision with a 
   //           turn is impossible! ===
   public float getCollisionRadius() {
      return -1.0f;
   }
   
   // Returns the id (also a ridiculous value)
   public String getId() {
      return "";
   }
   
   // Returns the x position
   public float getXPos() {
      return 0.0f;
   }
   
   // Returns the y position
   public float getYPos() {
      return 0.0f;
   }
 

}